

package asets.gui.application;

import java.awt.BorderLayout;
import java.awt.HeadlessException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.jms.InvalidClientIDException;
import javax.jms.JMSException;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import asets.api.Control;
import asets.api.Enums;
import asets.api.Execution;
import asets.api.InstrumentList;
import asets.api.Order;
import asets.api.OrderList;
import asets.api.User;
import asets.gui.communication.GUIExecutionSubscriber;
import asets.gui.communication.GUIInstrumentListSenderWithReply;
import asets.gui.communication.GUIOrderSubscriber;
import asets.gui.core.LaF;
import asets.gui.frames.MarketDepthFrame;
import asets.gui.menus.MainMenu;
import asets.gui.panels.OrdersPanel;
import asets.gui.panels.StatusBar;
import asets.gui.panels.WelcomePanel;

public class TradingGUI extends JPanel {

	private static final long serialVersionUID = 1L;
	private MainMenu mainMenu;
	private JPanel mainContentPanel;
	private StatusBar statusBar;
	private InstrumentList instrumentList;
	private Map<String, Integer> watchedSymbols = new HashMap<String, Integer>();

	private GUIOrderSubscriber orderSubscriber;
	private GUIExecutionSubscriber executionsSubscriber;
	private Long orderNumber;

	private IApplication container;

	/*
	 * JPanel containing the orders and executions tables
	 * */
	private OrdersPanel ordersAndExecutionsPanel;

	private User loggedUser;
	private String versionString;

	public static String IMQ_ADDRESS = "http://asets.ase.ro:8080/imqhttp/tunnel";
	//public static String IMQ_ADDRESS = "mq://127.0.0.1:7676/?instName=imqbroker";


	public TradingGUI( IApplication parent ) {
		super( new BorderLayout() );

		this.container = parent;
		this.init();
	}

	/*
	 * initialization
	 * */
	public void init() {

		System.setProperty("java.net.useSystemProxies", "true");

		LaF.setStartUpLookAndFeel();

		// attempt to read the version number
		InputStream is = getClass().getResourceAsStream("/META-INF/ASETS-VERSION.MF");
		
		// if there is a version file, load it
		if( is != null ) {
			try {
				Properties p = new Properties();
				p.load(is);
				versionString = p.getProperty("Implementation-Build");
				
			} catch (Exception e) {
				/* do nothing */
			}
		}

		orderNumber = Long.valueOf(0);
		loggedUser = null;

		setLayout(new BorderLayout()); 

		mainMenu = new MainMenu(this);
		container.setMenu(mainMenu);
		container.setTitle("ASETS");

		mainContentPanel = new JPanel();
		mainContentPanel.setLayout(new BorderLayout());
		add(mainContentPanel,BorderLayout.CENTER);

		statusBar = new StatusBar(this);
		add(statusBar,BorderLayout.SOUTH);
		setSize(800, 600);

		statusBar.setLabelText("You are not logged in.");

		//initialize the window status in "logged out" state
		logoutSuccessful();
	}

	public void destroy() {
		//close all connections to topics before exit
		closeAllSubscribers();
	}

	/**
	 * Returns the instrument list
	 * @return
	 */
	public InstrumentList getInstrumentList() {
		return instrumentList;
	}

	/**
	 * Replaces the content inside the main panel.
	 * @param panel the new panel to be used
	 */
	public void setContent(JPanel panel) {
		mainContentPanel.removeAll();
		mainContentPanel.add(panel, BorderLayout.CENTER);
	}

	/**
	 * Opens a popup frame with an error message.
	 * @param errorMessage the message to be shown
	 */
	public void showPopupError(String errorMessage) {
		JOptionPane.showMessageDialog(this,errorMessage);
	}


	/**
	 * To be called after a successful login. Activates the menu options, changes the main panel content, sets the current logged user.
	 * Subscribes to executions and order lists.
	 * @param user the logged in user
	 * @throws InvalidClientIDException 
	 */
	public void loginSuccessful(User user) throws InvalidClientIDException {
		setLoggedUser(user);
		ordersAndExecutionsPanel = new OrdersPanel(this);
		setContent(ordersAndExecutionsPanel);

		try {
			// subscribe for orders
			try {
				orderSubscriber = new GUIOrderSubscriber(this);
			} catch(InvalidClientIDException ex) {
				throw ex; // pass on
			}

			// subscribe for exception
			try {
				executionsSubscriber = new GUIExecutionSubscriber(this);
			} catch(InvalidClientIDException ex) {
				throw ex; // pass on
			}

		} catch(InvalidClientIDException ex) {
			throw ex; // pass on
		} catch(Exception ex) {
			showPopupError(ex.getMessage());
		}

		mainMenu.loginSuccessful();
		requestInstrumentList();
		statusBar.setConnected("You are logged in as "+ loggedUser.getUserName());
	}

	private void closeAllSubscribers() throws HeadlessException {
		if (orderSubscriber != null) {
			try {
				orderSubscriber.close();
			} catch (JMSException ex) {
				ex.printStackTrace();
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
		}
		if (executionsSubscriber != null) {
			try {
				executionsSubscriber.close();
			} catch (JMSException ex) {
				ex.printStackTrace();
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
		}

		Integer[] indexes = watchedSymbols.values().toArray(new Integer[0]);
		for(int index: indexes) {
			MarketDepthFrame f = this.mainMenu.getMarketDepthFrame(index);
			f.closeSubscriber();
		}
	}

	/**
	 * Retrieves the symbol list from OMS
	 */
	public void requestInstrumentList() {
		GUIInstrumentListSenderWithReply instrumentListSender = null;
		try {
			System.out.println("Loading instrument list...");
			Control instrumentControl = new Control(loggedUser.getUserID(), Enums.ControlType.INSTRUMENT_LIST_REQUEST,null);
			instrumentListSender = new GUIInstrumentListSenderWithReply(TradingGUI.IMQ_ADDRESS,
					Enums.QueueName.INSTRUMENT_LIST_REQUEST_QUEUE.toString(),
					Enums.QueueName.INSTRUMENT_LIST_REPLY_QUEUE.toString(),
					loggedUser.getUserID()+"-InstrumentsList");
			instrumentList = instrumentListSender.sendControlGetReply(instrumentControl);
			System.out.println("Instrument list retrieved.");
			
                        if (instrumentListSender != null)
                            instrumentListSender.close();
                        
		}
		catch(Exception e) {
			System.out.println("Instrument list retrieval failed. ");
			showPopupError(e.getMessage());
		}
		finally {
			if (instrumentList != null)
				try {
					instrumentListSender.close();
				}
			catch(Exception e1) {
				showPopupError(e1.getMessage());
			}
		}
	}

	/**
	 * To be called after the user logs out.
	 * Closes all subscribers, removes the main panel content and disables menu options.
	 */
	public void logoutSuccessful() {
		closeAllSubscribers();
		JPanel welcomePanel = new WelcomePanel(this);
		setContent(welcomePanel);
		setLoggedUser(null);
		mainMenu.logoutSuccessful();
		statusBar.setDisconnected("You are not logged in.");
		watchedSymbols.clear();
	}

	/*
	 * Restarts the session time
	 * */
	public void resetTime() {
		statusBar.restart();
	}


	/**
	 * Returns the current logged in user.
	 * @return
	 */
	public User getLoggedUser() {
		return loggedUser;
	}

	/**
	 * Sets a new logged in user. To be used after a successful login.
	 * @param user the new logged in user
	 */
	private void setLoggedUser(User user) {
		loggedUser = user;
	}

	/*
	 * Returns the current order list
	 * */
	public OrderList getOrderList() {
		return ordersAndExecutionsPanel.getOrderList();
	}

	/*
	 * Adds a new order to the order list
	 * */
	public void addOrder(Order order) {
		synchronized (ordersAndExecutionsPanel) {
			ordersAndExecutionsPanel.addOrder(order);
		}
	}

	/*
	 * Updates an order in order list based on orderID
	 * @param order the new order
	 * */
	public void updateOrder(Order order) {
		synchronized (ordersAndExecutionsPanel) {
			ordersAndExecutionsPanel.updateOrder(order);
		}
	}

	/*
	 * Searches trough the order list for the provided orderID.
	 * If found, the order is updated, else the new order is added at the end of the list.
	 * @param the new / updated order
	 * */
	public void addOrUpdateOrder(Order order) {
		synchronized (ordersAndExecutionsPanel) {
			ordersAndExecutionsPanel.addOrUpdateOrder(order);
		}
	}

	/*
	 * Searches trough the order list for the provided orderID.
	 * If found, the order is ignored, else the new order is added at the end of the list.
	 * @param the new order
	 * */
	public void addOrIgnoreOrder(Order order) {
		synchronized (ordersAndExecutionsPanel) {
			ordersAndExecutionsPanel.addOrIgnoreOrder(order);
		}
	}

	/*
	 * Adds a new execution at the end of the executions list
	 * @param execution the new execution to be added
	 * */
	public void addExecution(Execution execution) {
		ordersAndExecutionsPanel.addExecution(execution);
	}

	public Long getOrderNumber() {
		return orderNumber;
	}

	public void setOrderNumber(Long orderNumber) {
		this.orderNumber = orderNumber;
	}

	/**
	 * Marks a symbol as being watched by a MarketDepth window
	 * <p>Used for MarketDepth</p>
	 */
	public boolean watchSymbol(String symbol, int index) {
		if( watchedSymbols.containsKey(symbol) ) return false;
		watchedSymbols.put(symbol, index);
		return true;
	}

	/**
	 * Returns the window that monitors a specific symbol.
	 * The window is identified by it's internal index.
	 * <p>Used for MarketDepth</p>
	 */
	public MarketDepthFrame getSymbolWatcher(int index) {
		return mainMenu.getMarketDepthFrame(index);
	}

	/**
	 * Returns the window that monitors the specified symbol
	 * <p>Used for MarketDepth</p>
	 */
	public MarketDepthFrame getSymbolWatcher(String symbol) {
		Integer index = watchedSymbols.get(symbol);
		if( index == null ) return null;
		return mainMenu.getMarketDepthFrame(index.intValue());
	}

	/**
	 * <p>Used for MarketDepth</p>
	 */
	public void stopWatching(String symbol) {
		watchedSymbols.remove(symbol);
	}

	public int getWindowX() {
		return container.getWindowX();
	}

	public int getWindowY() {
		return container.getWindowY();
	}

	public int getWindowWidth() {
		return container.getWindowWidth();
	}

	public int getWindowHeight() {
		return container.getWindowHeight();
	}

	public String getVersionString() {
		return versionString;
	}

}
