/**
 * Title:       Comma-separated values Writer ( (ASE Trading System)
 * ClassName:   CSVWriter.java
 * Created:     January 22, 2010
 * Author:      Marius Stefan Paune
 * Email :      marius_paune@yahoo.com
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Formates a csv file
 */
package asets.gui.core;

import java.io.PrintWriter;

public final class CSVWriter {

    private static final String lineSeparator = "\r\n";
    private PrintWriter pw;
    /**
     * true if first field on line is ""
     */
    private boolean loneEmptyField;
    /**
     * true if write should trim lead/trail whitespace from fields before writing them.
     */
    private final boolean trim;
    /**
     * true if there has was a field previously written to this line, meaning there is a comma pending to be written.
     */
    private boolean wasPreviousField = false;
    /**
     * char to mark the start of a comment, usually ; or #
     */
    private char commentChar;
    /**
     * quote character, usually '\"' '\'' for SOL used to enclose fields containing a separator character.
     */
    private char quoteChar;
    /**
     * field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for tab.
     */
    private char separatorChar;
    /**
     * how much extra quoting you want
     */
    private int quoteLevel;

    public CSVWriter(PrintWriter pw) {
        // writer, quoteLevel, separatorChar, quoteChar, commentChar, trim
        this(pw, 1, ',', '\"', '#', true);
    }

    public CSVWriter(PrintWriter pw,
            int quoteLevel,
            char separatorChar,
            char quoteChar,
            char commentChar,
            boolean trim) {
        this.pw = pw;
        this.quoteLevel = quoteLevel;
        this.separatorChar = separatorChar;
        this.quoteChar = quoteChar;
        this.commentChar = commentChar;
        this.trim = trim;
    }

    /**
     * Close the PrintWriter.
     */
    public void close() {
        if (pw != null) {
            pw.close();
            pw = null;
        }
    }

    /**
     * Write a new line in the CVS output file to de-mark the end of record.
     */
    public void nl() {
        if (pw == null) {
            throw new IllegalArgumentException(
                    "attempt to write to a closed CSVWriter");
        }
        /* don't write last pending comma on the line */
        if (loneEmptyField) {
            // single empty field on line, currently displayed as 0 chars
            pw.write(quoteChar);
            pw.write(quoteChar);
        }

        pw.write(lineSeparator);
        wasPreviousField = false;
        loneEmptyField = false;
    }

    public void nl(final String comment) {
        if (pw == null) {
            throw new IllegalArgumentException(
                    "attempt to write to a closed CSVWriter");
        }
        if (wasPreviousField) {
            if (loneEmptyField) {
                // single empty field on line, currently displayed as 0 chars
                pw.write(quoteChar);
                pw.write(quoteChar);
            }
            // no comma, just extra space.
            pw.write(' ');
        }

        pw.write(commentChar);  // start comment with space # space
        pw.write(' ');
        pw.write(comment.trim());
        pw.write(lineSeparator);
        wasPreviousField = false;
        loneEmptyField = false;
    }

    /**
     * Write one csv field to the file, followed by a separator unless it is the last field on the line. Lead and
     * trailing blanks will be removed.
     *
     * @param i The int to write. Any additional quotes or embedded quotes will be provided by put.
     */
    public void put(int i) {
        put(Integer.toString(i));
    }

    /**
     * Write one csv field to the file, followed by a separator unless it is the last field on the line. Lead and
     * trailing blanks will be removed.
     *
     * @param c The char to write. Any additional quotes or embedded quotes will be provided by put.
     */
    public void put(char c) {
        put(String.valueOf(c));
    }

    /**
     * Write one csv field to the file, followed by a separator unless it is the last field on the line. Lead and
     * trailing blanks will be removed.
     *
     * @param l The long to write. Any additional quotes or embedded quotes will be provided by put.
     */
    public void put(long l) {
        put(Long.toString(l));
    }

    public void put(double d) {
        put(Double.toString(d));
    }

    public void put(float f) {
        put(Float.toString(f));
    }

    public void put(String s) {
        if (pw == null) {
            throw new IllegalArgumentException(
                    "attempt to write to a closed CSVWriter");
        }
        if (s == null) {
            nl();
            return;
        }

        if (trim) {
            s = s.trim();
        }
        if (wasPreviousField) {
            pw.write(separatorChar);
            if (quoteLevel == -1) {
                pw.write(' ');
            }
            loneEmptyField = false;
        } else {
            // first field on line
            loneEmptyField = s.trim().length() == 0;
        }
        if (s.indexOf(quoteChar) >= 0) {
            /* worst case, needs surrounding quotes and internal quotes doubled */
            pw.write(quoteChar);
            for (int i = 0, n = s.length(); i < n; i++) {
                char c = s.charAt(i);
                if (c == quoteChar) {
                    pw.write(quoteChar);
                    pw.write(quoteChar);
                } else {
                    pw.write(c);
                }
            }
            pw.write(quoteChar);
        } // no internal quotes to worry about.
        else if (quoteLevel == 2 || quoteLevel == 1 && s.indexOf(' ') >= 0 || s.indexOf(separatorChar) >= 0 || s.indexOf(commentChar) >= 0) {
            /* need surrounding quotes */
            pw.write(quoteChar);
            pw.write(s);
            pw.write(quoteChar);
        } else {
            /* ordinary case, no surrounding quotes needed */
            pw.write(s);
        }
        /* make a note to print trailing comma later */
        wasPreviousField = true;
    }
}// end CSVWriter class.
