/**
 * Title:       GUI Helper class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Provides a various functionality for GUI
 */
package asets.gui.core;

import javax.swing.*;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.prefs.Preferences;

import static java.lang.System.err;

/**
 *
 * @author NSG
 */
public final class LaF {

    public static String persistancePath = "/asets/gui/core/laf";
    public static String persitanceLaFName = "AssetsLaFName";
    public static String persitanceLaFClassName = "AssetsLaFClassName";
    public static String menuName = "Skins";
    public static char menuAcc = 'k';
    // -------------------------- PUBLIC STATIC METHODS --------------------------

    /**
     * build JMenu to allow user to choose a L&F. It must be installed in a JMenuBar.
     *
     * @return the JMenu containing all the supported Look and Feels
     */
    public static JMenu buildLookAndFeelMenu() {
        // anti-alias fonts
        //System.setProperty( "swing.aatext", "true" );
        final JMenu menuLAF = new JMenu(LaF.menuName);
        menuLAF.setMnemonic(LaF.menuAcc);
        ActionListener changeLAFListener = new ActionListener() {

            public void actionPerformed(final ActionEvent e) {
                final JRadioButtonMenuItem rbmi = (JRadioButtonMenuItem) e.getSource();
                setLookAndFeel(rbmi.getActionCommand());
                propagateLookAndFeelChange();
                persistLookAndFeel(new UIManager.LookAndFeelInfo(rbmi.getText(), rbmi.getActionCommand()));
            }
        };

        UIManager.LookAndFeelInfo laf = LaF.getPersistedLookAndFeel();
        if (laf == null) {
            laf = LaF.getSystemLookAndFeel();
            if (laf == null) {
                laf = LaF.getPreferredLookAndFeel();
                if (laf == null) {
                    laf = LaF.getCurrentLookAndFeel();
                }
            }
        }

        final ButtonGroup bg = new ButtonGroup();
        final UIManager.LookAndFeelInfo[] installed = UIManager.getInstalledLookAndFeels();

        for (UIManager.LookAndFeelInfo info : installed) {
            final JRadioButtonMenuItem rbmi = new JRadioButtonMenuItem(info.getName(), isSameLookAndFeel(info, laf));
            menuLAF.add(rbmi);
            bg.add(rbmi);
            rbmi.setActionCommand(info.getClassName());
            rbmi.addActionListener(changeLAFListener);
        }
        return menuLAF;
    }

    /**
     * get current L&F
     *
     * @return L&F currently in effect
     */
    public static UIManager.LookAndFeelInfo getCurrentLookAndFeel() {
        final LookAndFeel current = UIManager.getLookAndFeel();
        final String currentLAFName = current.getName();
        final String currentLAFClassName = current.getClass().getName();

        if (currentLAFName.length() > 0) {
            return new UIManager.LookAndFeelInfo(currentLAFName, currentLAFClassName);
        } else {
            return null;
        }
    }

    /**
     * fetch the persisted L&F the user has selected previously
     *
     * @return L&F user chose last time, null if none.
     */
    public static UIManager.LookAndFeelInfo getPersistedLookAndFeel() {
        final Preferences lafPref = Preferences.userRoot().node(persistancePath);

        final String persistedLAFName = lafPref.get(LaF.persitanceLaFName, "");
        final String persistedLAFClassName = lafPref.get(LaF.persitanceLaFClassName, "");
        if (persistedLAFName.length() > 0) {
            final UIManager.LookAndFeelInfo laf = new UIManager.LookAndFeelInfo(persistedLAFName, persistedLAFClassName);
            if (isLookAndFeelSupported(laf)) {
                return laf;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    /**
     * Get the preferred LookAndFeel, default before the user has expressed a preference.
     *
     * @return look and feel, null if none of desirable ones are supported.
     */
    public static UIManager.LookAndFeelInfo getPreferredLookAndFeel() {
        // sorted in order with most preferable first.
        final String preferredClassNames[] = {"javax.swing.plaf.metal.MetalLookAndFeel", // aka cross platform
            "com.sun.java.swing.plaf.mac.MacLookAndFeel",
            "com.sun.java.swing.plaf.windows.WindowsLookAndFeel",
            "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel",
            "com.sun.java.swing.plaf.gtk.GTKLookAndFeel",
            "com.sun.java.swing.plaf.motif.MotifLookAndFeel",
            UIManager.getSystemLookAndFeelClassName()};
        // find best/first L&F that is supported.
        final UIManager.LookAndFeelInfo[] installed = UIManager.getInstalledLookAndFeels();
        for (final String preferredClassName : preferredClassNames) {
            for (UIManager.LookAndFeelInfo anInstalled : installed) {
                if (preferredClassName.equals(anInstalled.getClassName())) {
                    return new UIManager.LookAndFeelInfo(anInstalled.getName(), preferredClassName);
                }
            }
        }
        return null;
    }

    /**
     * Test if this L&F is supported
     *
     * @param laf to check if is supported
     *
     * @return true laf is supported.
     */
    public static boolean isLookAndFeelSupported(UIManager.LookAndFeelInfo laf) {
        final UIManager.LookAndFeelInfo[] installed = UIManager.getInstalledLookAndFeels();
        // can't use for:each in jdk 1.3
        for (UIManager.LookAndFeelInfo info : installed) {
            if (isSameLookAndFeel(info, laf)) {
                return true;
            }
        }
        return false;
    }

    /**
     * save L&F using the persistence mechanism
     *
     * @param laf L&F to persist
     */
    public static void persistLookAndFeel(UIManager.LookAndFeelInfo laf) {
        // requires JDK 1.4+
        Preferences lafPref = Preferences.userRoot().node(LaF.persistancePath);
        lafPref.put(LaF.persitanceLaFName, laf.getName());
        lafPref.put(LaF.persitanceLaFClassName, laf.getClassName());
    }

    /**
     * make sure all Frames and Windows are repainted with the new Look and Feel.
     */
    public static void propagateLookAndFeelChange() {
        // propagate new L&F to all frames.
        final Frame frames[] = Frame.getFrames();
        if (frames.length == 0) {
            return;
        }

        // refresh all Frames in the app
        for (Frame frame : frames) {
            SwingUtilities.updateComponentTreeUI(frame);
            Window windows[] = frame.getOwnedWindows();

            // refresh all windows and dialogs of the frame
            for (Window window : windows) {
                SwingUtilities.updateComponentTreeUI(window);
            }
        }

        // Restore decoration to outermost JFrame.
        try {
            final JFrame outerFrame = (JFrame) frames[ 0];
            outerFrame.setVisible(false);
            outerFrame.dispose();
            outerFrame.setUndecorated(false);
            outerFrame.getRootPane().setWindowDecorationStyle(JRootPane.NONE);
            outerFrame.setVisible(true);
        } catch (ClassCastException e) {
            // was an JApplet or AppletViewer as the outer frame. Does not have a bar to decorate.
        }
    }

    /**
     * change the current look and feel.
     * If the class is invalid, the command is ignored.
     *
     * @param lookAndFeelClassName name of Look and Feel class
     */
    public static void setLookAndFeel(String lookAndFeelClassName) {
        // if already set, nothing to do
        if (UIManager.getLookAndFeel().getClass().getName().equals(lookAndFeelClassName)) {
            return;
        }
        try {
            UIManager.setLookAndFeel(lookAndFeelClassName);
        } catch (Exception exception) {
            err.println("Setting Look and Feel failed");
            err.println(exception.getMessage());
        }
    }

     /**
     * Gets the system look and feel.

     */
    public static UIManager.LookAndFeelInfo getSystemLookAndFeel() {
        String systemLaFClass = UIManager.getSystemLookAndFeelClassName();
        final UIManager.LookAndFeelInfo[] installed = UIManager.getInstalledLookAndFeels();

        for (UIManager.LookAndFeelInfo anInstalled : installed) {
            if (systemLaFClass.equals(anInstalled.getClassName())) {
                return new UIManager.LookAndFeelInfo(anInstalled.getName(), systemLaFClass);
            }
        }

        return null;
    }

    public static void setStartUpLookAndFeel() {
        UIManager.LookAndFeelInfo laf = LaF.getPersistedLookAndFeel();
        if (laf == null) {
            laf = LaF.getSystemLookAndFeel();
            if (laf == null) {
                laf = LaF.getPreferredLookAndFeel();
                if (laf == null) {
                    laf = LaF.getCurrentLookAndFeel();
                }
            }
        }
        setLookAndFeel(laf.getClassName());
        // we don't do a propagateLookAndFeelChange since the GUI is not realised yet.
    }

    // -------------------------- PRIVATE STATIC METHODS --------------------------
    /**
     * Test if two L&Fs match
     *
     * @param a first L&F
     * @param b second L&F
     *
     * @return true if both the name and className match
     */
    private static boolean isSameLookAndFeel(UIManager.LookAndFeelInfo a, UIManager.LookAndFeelInfo b) {
        if (a == b) {
            return true;
        }
        return a != null && b != null && a.getName().equals(b.getName()) && a.getClassName().equals(b.getClassName());
    }
}
