/**
 * Title:       GUI Frame class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements the functionality for a component in GUI
 */

package asets.gui.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SpringLayout;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import asets.api.Instrument;
import asets.api.InstrumentList;
import asets.gui.application.TradingGUI;
import asets.gui.core.CSVWriter;
import asets.gui.core.UseFileDialog;

public class InstrumentsFrame extends JFrame implements ActionListener{
	private static final long serialVersionUID = 1L;
	private final String[] tableColumns = new String[]{"Symbol","Name","Segment","Type","MainMarket","Tier","Status","LastClosePrice","CurrentPrice","LastUpdate"};
	
	private TradingGUI parentApplet;
	private JButton buttonRefresh, buttonSave, buttonClose;
	
	private JTable instrumentsTable;
	private DefaultTableModel instrumentTableModel;
	private InstrumentList instrumentList;
	private Object[][] instrList;
	
	private JPanel loadingPanel;
	private JPanel footerPanel;
	private JScrollPane instrTableScrollPane;


	public InstrumentsFrame(TradingGUI parentApplet) {
		this.parentApplet = parentApplet;
		
		this.setTitle("Instruments");
		setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
		setDefaultCloseOperation(HIDE_ON_CLOSE); // this will allow the listener to dispose of this
		setResizable(true);
		
		initComponents();
		setLocationRelativeTo(parentApplet);
		
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				loadInstruments();
			}
		});

	}
	
	private void loadInstruments() {
		parentApplet.requestInstrumentList();
		instrumentList = parentApplet.getInstrumentList();
		
		int nbSymbols = instrumentList.getInstrumentList().size();
		instrList = new Object [nbSymbols][10];

		int i = 0;
		for(String symbol: instrumentList.getInstrumentList().keySet()) {
			instrList[i++][0] = symbol;
		}
		
		i = 0;
		for(Instrument instr: instrumentList.getInstrumentList().values()) {
			instrList[i][1] = instr.getName();
			instrList[i][2] = instr.getSegment();
			instrList[i][3] = instr.getType();
			instrList[i][4] = instr.getMainMarket();
			instrList[i][5] = instr.getTier();
			instrList[i][6] = instr.getStatus();
			instrList[i][7] = instr.getLastClosePrice();
			instrList[i][8] = instr.getCurrentPrice();
			instrList[i][9] = instr.getLastUpdate();

			i++;
		}
		
		instrumentTableModel.setDataVector(instrList, tableColumns);
		loadingPanel.setVisible(false);
	}
	
	public void initComponents() {
		
		buttonRefresh = new JButton("Refresh");
		buttonSave = new JButton("Export");
		buttonClose = new JButton("Close");
		
		buttonClose.addActionListener(this); 	buttonRefresh.setMnemonic('f');
		buttonSave.addActionListener(this); 	buttonSave.setMnemonic('e');
		buttonRefresh.addActionListener(this); 	buttonClose.setMnemonic('c');

		instrumentTableModel = new DefaultTableModel(instrList, tableColumns) {
			private static final long serialVersionUID = 1L;
			@Override
			public boolean isCellEditable(int x, int y) {
				return false;
			}
			
			@Override
			public Class<?> getColumnClass(int column) {
				Class<?> c = null;
				switch(column) {
				case 7: // fall-through
				case 8: c = Float.class; break;
				case 9: c = Date.class; break;
				default: c = String.class;
				}
				return c;
			}
		};

		instrumentsTable = new JTable(instrumentTableModel){
			private static final long serialVersionUID = 1L;

			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
				Component c = super.prepareRenderer(renderer, row, column);
				if (!c.getBackground().equals(getSelectionBackground())) {
					c.setBackground( new Color(223,255,146) );
				}
				return c;
			}
		};
		instrTableScrollPane = new JScrollPane(instrumentsTable);

		instrumentsTable.setName("instrumentsTable"); 
		instrumentsTable.getTableHeader().setReorderingAllowed(true);
		instrumentsTable.setAutoCreateRowSorter(true);

		footerPanel = new javax.swing.JPanel();
		loadingPanel = new javax.swing.JPanel();
		JPanel buttonPanel = new JPanel();
		SpringLayout layout = new SpringLayout();

		buttonPanel.add(buttonRefresh);
		buttonPanel.add(buttonSave);
		buttonPanel.add(buttonClose);
		
		loadingPanel.add(new JLabel("Loading... Please wait..."));
		
		footerPanel.setLayout(layout);
		footerPanel.setPreferredSize(new java.awt.Dimension(800, 35));
		footerPanel.add(loadingPanel, BorderLayout.WEST);
		footerPanel.add(buttonPanel, BorderLayout.CENTER);

		layout.putConstraint(SpringLayout.NORTH, loadingPanel, 5, SpringLayout.NORTH, footerPanel);
		layout.putConstraint(SpringLayout.EAST, buttonPanel, 0, SpringLayout.EAST, footerPanel);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(instrTableScrollPane,BorderLayout.CENTER);
		getContentPane().add(footerPanel,BorderLayout.SOUTH);
		
		pack();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		JButton button = (JButton)e.getSource();
		
		if(button == buttonRefresh){
			loadingPanel.setVisible(true);
			SwingUtilities.invokeLater(new Runnable() {
				
				@Override
				public void run() {
					loadInstruments();
				}
			});
			
		} else if(button == buttonClose) {
			this.setVisible(false);
			
		} else if(button == buttonSave){

			CSVWriter csv = null;
			String name = UseFileDialog.saveFile(this, "Export instruments...", ".", "instruments.csv");
			if( name == null || name.length() == 0 ) return;

			if( !name.toLowerCase().endsWith(".csv") && !name.toLowerCase().endsWith(".txt") && !name.toLowerCase().endsWith(".tsv") ) {
				name = name + ".csv";
			}

			try {

				csv = new CSVWriter(new PrintWriter(new BufferedWriter(new FileWriter(name))), 1, ',', '\"', '#', true);

				// header
				for (int i = 0; i < instrumentsTable.getColumnCount(); i++) {
					csv.put(instrumentsTable.getColumnName(i));
				}
				csv.nl();

				// data
				for (int j = 0; j < instrumentsTable.getRowCount() - 1; j++) {
					for (int i = 0; i < instrumentsTable.getColumnCount(); i++) {
						csv.put(instrumentsTable.getValueAt(j, i).toString());
					}
					csv.nl();
				}


			} catch (IOException ex) {
				parentApplet.showPopupError(ex.getMessage());

			} finally {
				try { csv.close(); }
				catch(Exception ex) { /* do nothing */ }
			}

		}
	}


}
