package asets.gui.frames;

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DecimalFormat;
import java.util.ArrayDeque;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.locks.ReentrantLock;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import asets.api.Control;
import asets.api.Enums;
import asets.api.Instrument;
import asets.api.InstrumentList;
import asets.api.Order;
import asets.api.OrderList;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUIInstrumentOrderListSenderWithReply;
import asets.gui.communication.GUIInstrumentOrderSubscriber;
import asets.gui.core.InstrumentPriceInfo;
import asets.gui.core.PriceFormat;


public class MarketDepthFrame extends JFrame {

	private static final long serialVersionUID = 1L;
	private static final String[] tableColumns = {"# BID", "Vol BID", "Price","Vol ASK","# ASK"};

        private final static Float BEST_BUY = 9999999F;
        private final static Float BEST_SELL = 0.0F;
    
	private int frameCount = 0;
	private volatile int middleRow = 0;
	private final ReentrantLock tableUpdate = new ReentrantLock();

	private javax.swing.JTable ordersTable;
	private javax.swing.JComboBox symbolComboBox;
	private javax.swing.JScrollPane topScrollPane;

	private TradingGUI parentApplet;
	private InstrumentList instrumentList;
	private OrderList orderList;

	protected DefaultTableModel ordersTableModel;
	private SortedMap<Float,InstrumentPriceInfo> buyTreeMap, sellTreeMap;
	private Map<String, Order> allOrders;
	private String selectedSymbol = null;

	private GUIInstrumentOrderSubscriber orderSubscriber = null;


	public MarketDepthFrame(TradingGUI parentApplet, int cnt) {
		this.parentApplet = parentApplet;
		this.frameCount = cnt;

		setTitle("Market Depth");
		setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
		setDefaultCloseOperation(HIDE_ON_CLOSE); // this will allow the listener to dispose of this
		setResizable(true);

		initComponents();
		setLocationRelativeTo(parentApplet);

		SwingUtilities.invokeLater( new Runnable() {
			@Override
			public void run() {
				populateSymbolComboBox();
			}
		});

	}


	private void initComponents() {

		javax.swing.JPanel mainPanel = new javax.swing.JPanel();
		javax.swing.JPanel controlsPanel = new javax.swing.JPanel();
		javax.swing.JPanel dataPanel = new javax.swing.JPanel();

		javax.swing.JLabel symbolLabel = new javax.swing.JLabel();
		javax.swing.JButton viewButton = new javax.swing.JButton();
		javax.swing.JLabel ordersLabel = new javax.swing.JLabel();
		symbolComboBox = new javax.swing.JComboBox();
		topScrollPane = new javax.swing.JScrollPane();
		ordersTable = new javax.swing.JTable();

		mainPanel.setMinimumSize(new java.awt.Dimension(130, 464));
		mainPanel.setPreferredSize(new java.awt.Dimension(300, 464));
		mainPanel.setLayout(new java.awt.BorderLayout());


		symbolLabel.setFont(new java.awt.Font("Tahoma", 1, 11));
		symbolLabel.setText("Symbol:");
		symbolLabel.setPreferredSize(new java.awt.Dimension(70, 14));

		symbolComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
		symbolComboBox.setPreferredSize(new java.awt.Dimension(100, 22));

		controlsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		controlsPanel.add(symbolLabel);
		controlsPanel.add(symbolComboBox);

		viewButton.setText("View");
		viewButton.setMnemonic('v');
		viewButton.setPreferredSize(new java.awt.Dimension(70, 22));
		viewButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				changeSymbol();
			}
		});

		controlsPanel.add(viewButton);
		mainPanel.add(controlsPanel, java.awt.BorderLayout.NORTH);


		ordersLabel.setFont(new java.awt.Font("Tahoma", 1, 11));
		ordersLabel.setText("  Market Orders:");
		ordersLabel.setAutoscrolls(true);
		ordersLabel.setFocusable(false);
		ordersLabel.setMaximumSize(new java.awt.Dimension(800, 14));
		ordersLabel.setPreferredSize(new java.awt.Dimension(800, 30));

		ordersTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		ordersTable.getTableHeader().setReorderingAllowed(false);
		topScrollPane.setViewportView(ordersTable);

		dataPanel.setLayout(new java.awt.BorderLayout());
		dataPanel.add(ordersLabel, java.awt.BorderLayout.NORTH);
		dataPanel.add(topScrollPane, java.awt.BorderLayout.CENTER);

		mainPanel.add(dataPanel, java.awt.BorderLayout.CENTER);
		getContentPane().add(mainPanel, java.awt.BorderLayout.CENTER);

		orderSubscriber = null;
		
		buyTreeMap = new TreeMap<Float, InstrumentPriceInfo>();
		sellTreeMap = new TreeMap<Float, InstrumentPriceInfo>();
		allOrders = new HashMap<String, Order>();


		ordersTableModel = new DefaultTableModel(new Object[0][0], tableColumns) {
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int x, int y) {
				return false;
			}

			@Override
			public Class<?> getColumnClass(int column) {
				if( column == 2 ) return Float.class;
				return Integer.class;
			}
		};

		ordersTable = new JTable(ordersTableModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
				Component c = null;
				tableUpdate.lock();

				try {
					c = super.prepareRenderer(renderer, row, column);
					if (!c.getBackground().equals(getSelectionBackground())) {
						try {
							if( ordersTableModel.getValueAt(row, 1) != null ) {
								// this is buy
								c.setBackground( new Color(223,255,146) );
							} else {
								// this is sell
								c.setBackground( new Color(255,205,216) );
							}
						} catch( Exception e ) {
							e.printStackTrace();
							c.setBackground( new Color(255,205,216) );
						}
					}
				} finally {
					tableUpdate.unlock();
				}

				return c;
			}
		};

		ordersTable.setDefaultRenderer(Float.class, new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				((JLabel)c).setHorizontalAlignment(SwingConstants.CENTER);
				if (value != null) {
					if (value.getClass() == Float.class) {

						DecimalFormat twoDForm = new PriceFormat();
						((JLabel)c).setText(twoDForm.format((Float)value));

						if (column == 2 && ( ((Float)value).equals(BEST_BUY) || ((Float)value).equals(BEST_SELL) ) ) {
							((JLabel)c).setText("M.P.");
						}
					}
				}
				return c;
			}

		});


		topScrollPane.setViewportView(ordersTable);

		this.addWindowListener(new WindowAdapter() {
                    @Override
                    public void windowClosing(WindowEvent e) 
                    {
                        closeSubscriber();
                    }
		});

		pack();
	}


	/**
	 * Populates the symbol drop-down with the values from the server
	 */
	private void populateSymbolComboBox() {
		instrumentList = parentApplet.getInstrumentList();
		if (instrumentList != null) {
			Collection<Instrument> c = instrumentList.getInstrumentList().values();
			String[] symbols = new String[c.size()];
			int i=0;
			Iterator<Instrument> it = c.iterator();
			while (it.hasNext()) {
				symbols[i] = it.next().getSymbol();
				i++;
			}
			ComboBoxModel cbM = new DefaultComboBoxModel(symbols);
			symbolComboBox.setModel(cbM);
		}
	}


	/**
	 * Utility function for changing the symbol that is being watched
	 */
	private void changeSymbol() {
            String symbol = symbolComboBox.getSelectedItem().toString();

            if( !symbol.equals(selectedSymbol) ) {
                    boolean symbolAvailable = parentApplet.watchSymbol(symbol, frameCount);

                    if( !symbolAvailable ) {
                            // this symbol is already watched from another window
                            JOptionPane.showMessageDialog(this,"Symbol " + symbol + " is already being monitored in another window. Please choose another symbol.");
                            parentApplet.getSymbolWatcher(symbol).setVisible(true);
                            return;

                    } else {
                            // symbol can be selected - unregister previous symbol
                            if( selectedSymbol != null ) {
                                    parentApplet.stopWatching(selectedSymbol);
                            }
                    }

                    selectedSymbol = symbol;
                    this.setTitle("Market Depth - " + symbol);
            }

            buyTreeMap.clear();
            sellTreeMap.clear();
            allOrders.clear();

            //load orders....
            System.out.println("Loading market orders...");
            orderList = null;
            GUIInstrumentOrderListSenderWithReply orderListSender = null;
            String userID = parentApplet.getLoggedUser().getUserID();

            try {
                Control control = new Control(userID, Enums.ControlType.INSTRUMENT_ORDER_LIST_REQUEST, selectedSymbol);
                orderListSender = new  GUIInstrumentOrderListSenderWithReply(TradingGUI.IMQ_ADDRESS,
                        Enums.QueueName.INSTRUMENT_ORDER_LIST_REQUEST_QUEUE.toString(),
                        Enums.QueueName.INSTRUMENT_ORDER_LIST_REPLY_QUEUE.toString(),
                        userID);
                orderList = orderListSender.sendControlGetReply(control);
            }
            catch (Exception e){
                parentApplet.showPopupError(e.getMessage());
            }
            finally {
                try { 
                    if (orderListSender!=null) 
                        orderListSender.close(); 
                } catch (Exception ex) { 
                    parentApplet.showPopupError(ex.getMessage()); 
                }
            }


            if (orderList != null) {
                System.out.println("Received orders...");
                ArrayDeque<Order> orders = orderList.getOrderList();
                Iterator<Order> o = orders.iterator();
                while( o.hasNext()) {
                    Order order = o.next();
                    if( order.getOrderQuant() == 0 ) continue;
                    allOrders.put(order.getOrderID(), order);

                    if (order.getOrderSide().equals(Enums.Side.BUY)) {
                        InstrumentPriceInfo priceInfo = buyTreeMap.get(order.getOrderPrice());
                        if (priceInfo == null) {
                            priceInfo = new InstrumentPriceInfo(order.getOrderPrice());
                        }
                        priceInfo.addOrder(order);
                        buyTreeMap.put(order.getOrderPrice(), priceInfo);
                    }
                    else {
                        InstrumentPriceInfo priceInfo = sellTreeMap.get(order.getOrderPrice());
                        if (priceInfo == null) {
                            priceInfo = new InstrumentPriceInfo(order.getOrderPrice());
                        }
                        priceInfo.addOrder(order);
                        sellTreeMap.put(order.getOrderPrice(), priceInfo);
                    }
                }

                repopulateTable();
            }

            if (orderSubscriber != null) 
            {
                try { 
                    orderSubscriber.close();
                } catch(Exception e) { e.printStackTrace(); }
            }

            try {
                orderSubscriber = new GUIInstrumentOrderSubscriber(parentApplet, this, selectedSymbol);
            } catch(Exception e) { e.printStackTrace(); }
    }


    /**
     * This is executed when a new order is received on the order topic
     * <p>The new order is placed in a collection and then the table is
     * repainted</p>
     * 
     * @param order The newly received order
     * 
     * @see #repopulateTable()
     */
    public void receivedOrder(Order order) {
            // only one thread should access this method at a time
            tableUpdate.lock();

            try {
                if (order.getOrderSide().equals(Enums.Side.BUY)) {

                    // if this order was updated, verify if price has changed
                    if( order.getOrderStatus().equals(Enums.OrderStatus.UPDATED )) {
                        Order existing = allOrders.get(order.getOrderID());
                        if (existing != null)
                        {
                            if( existing.getOrderPrice() != order.getOrderPrice() ) {
                                // price has changed, remove order from old position
                                InstrumentPriceInfo ipr = buyTreeMap.get(existing.getOrderPrice()); // old price
                                if( ipr != null ) {
                                    // remove order
                                    ipr.removeOrder( existing.getOrderID() );

                                    // if nothing remained on that position, simply clear it
                                    if( ipr.getQuantity() == 0 ) buyTreeMap.remove(existing.getOrderPrice());
                                }
                            }
                        }
                    }

                    // add or update this in the order list
                    allOrders.put(order.getOrderID(), order);

                    // insert this order on its corresponding position
                    InstrumentPriceInfo priceInfo = buyTreeMap.get(order.getOrderPrice());
                    if (priceInfo == null) priceInfo = new InstrumentPriceInfo(order.getOrderPrice());
                    priceInfo.addOrder(order);

                    // verify quantity of that position
                    if (priceInfo.getQuantity() > 0) {
                        buyTreeMap.put(order.getOrderPrice(), priceInfo);
                    } else {
                        buyTreeMap.remove(order.getOrderPrice());
                        allOrders.remove(order.getOrderID());
                    }

                } else {

                    // if this order was updated, verify if price has changed
                    if( order.getOrderStatus() == Enums.OrderStatus.UPDATED ) {
                        Order existing = allOrders.get(order.getOrderID());
                        if (existing != null)
                        {
                            if( existing.getOrderPrice() != order.getOrderPrice() ) {
                                // price has changed, remove order from old position
                                InstrumentPriceInfo ipr = sellTreeMap.get(existing.getOrderPrice()); // old price
                                if( ipr != null ) {
                                    // remove order
                                    ipr.removeOrder( existing.getOrderID() );

                                    // if nothing remained on that position, simply clear it
                                    if( ipr.getQuantity() == 0 ) sellTreeMap.remove(existing.getOrderPrice());
                                }
                            }
                        }
                    }

                    // add or update this in the order list
                    allOrders.put(order.getOrderID(), order);

                    // insert this order on its corresponding position
                    InstrumentPriceInfo priceInfo = sellTreeMap.get(order.getOrderPrice());
                    if (priceInfo == null) priceInfo = new InstrumentPriceInfo(order.getOrderPrice());
                    priceInfo.addOrder(order);

                    // verify quantity of that position
                    if (priceInfo.getQuantity() > 0) {
                        sellTreeMap.put(order.getOrderPrice(), priceInfo);
                    } else {
                        sellTreeMap.remove(order.getOrderPrice());
                        allOrders.remove(order.getOrderID());
                    }
                }

                repopulateTable();

            } finally {
                tableUpdate.unlock();
            }
	}


	/**
	 * This is called when the order data has changed and the table needs to be reloaded.
	 * <p>This method should not be accessed concurrently (by more than one thread at a time).
	 * Given the fact that the repainting of the table is handled by the JVM and it's done
	 * in a separate thread this method cannot be declared as <i>synchronized</i> (the calls
	 * to this method would be synchronized, but the repainting of the table as a result of
	 * this method call, however will not be). </p>
	 * <p>Therefore the synchronization is done explicitly using a Lock object - the lock is
	 * acquired on this method call as well as on repainting, making sure no repaint is done
	 * while the table data is changed</p>
	 */
	private void repopulateTable() {

		ordersTableModel.setRowCount(0);

		int i = 0;
		Iterator<InstrumentPriceInfo> s = sellTreeMap.values().iterator();
		for (; s.hasNext();) {
			Object[] data = new Object[5];
			InstrumentPriceInfo sellPriceInfo = s.next();
			if(sellPriceInfo.getQuantity() > 0 && sellPriceInfo.getNumOrders() > 0){
				i++;
				data[2] = (float) sellPriceInfo.getPrice();
				data[3] = sellPriceInfo.getQuantity();
				data[4] = sellPriceInfo.getNumOrders();
				ordersTableModel.insertRow(0, data);
			}
		}

		middleRow = i;

		Iterator<InstrumentPriceInfo> b = ((TreeMap<Float,InstrumentPriceInfo>) buyTreeMap).descendingMap().values().iterator();
		for (; b.hasNext();) {
			Object[] data = new Object[5];
			InstrumentPriceInfo buyPriceInfo = b.next();
			if(buyPriceInfo.getQuantity() > 0 && buyPriceInfo.getNumOrders() > 0){
				data[0] = buyPriceInfo.getNumOrders();
				data[1] = buyPriceInfo.getQuantity();
				data[2] = (float) buyPriceInfo.getPrice();
				ordersTableModel.addRow(data);
			}
		}

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				scrollToVisible();
			}
		});

	}


	/**
	 * This function is used to scroll the table to the middle
	 * row (the row when ASK meets BID).
	 * <p>This is called after loading the orders or whenever a new
	 * order is added to the table.</p>
	 */
	public void scrollToVisible() {
		JTable table = ordersTable;
		int rowIndex = middleRow + 12;

		JViewport viewport = (JViewport)table.getParent();

		// The location of the view-port relative to the table
		Point pt = viewport.getViewPosition();

		// Scroll to top, first
		Rectangle top = new Rectangle(1,1);
		top.setLocation(-pt.x, -pt.y);
		viewport.scrollRectToVisible(top);

		// This rectangle is relative to the table where the
		// northwest corner of cell (0,0) is always (0,0).
		Rectangle rect = table.getCellRect(rowIndex, 0, true);

		// Scroll the area into view
		viewport.scrollRectToVisible(rect);
	}


	/**
	 * This should be called when the MarketDepth window is closed to free up the
	 * subscriber connection.
	 */
	public final void closeSubscriber() {
		if (orderSubscriber != null) {
			try {
				parentApplet.stopWatching(selectedSymbol);
				orderSubscriber.close();
			} catch(Exception ex) {
				ex.printStackTrace();
			}
		}
	}


	/**
	 * Returns the index of the current window
	 * @return an integer value
	 */
	public int getFrameCount() {
		return frameCount;
	}
}
