/**
 * Title:       Portfolio (ASE Trading System)
 * ClassName:   Portofolio.java
 * Created:     January 22, 2010
 * Author:      Marius Stefan Paune / Adrian Bardan / Ionut-Alexandru Lixandru / Claudiu Vinte 
 * Email:       claudiu.vinte@ie.ase.ro
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Main page for consulting user portfolio
 */
package asets.gui.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Date;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SpringLayout;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import asets.api.Control;
import asets.api.Enums;
import asets.api.Instrument;
import asets.api.InstrumentList;
import asets.api.Portfolio;
import asets.api.PortfolioList;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUIPortfolioListSenderWithReply;
import asets.gui.core.CSVWriter;
import asets.gui.core.PriceFormat;
import asets.gui.core.UseFileDialog;

/**
 *
 * @author Marius
 */
public class PortfolioFrame extends JFrame {

	private static final long serialVersionUID = 1L;
	private static final String[] tableColumns = new String[]{"Symbol", "Last Update", "Aquisition Quantity ", "AVG Price ", "Current Price", "Aquisition Value", "Current value", "Net Changed", "% Change"};

	private String userID;
	private TradingGUI parentApplet;

	private javax.swing.JPanel loadingPanel;
	private javax.swing.JPanel footerPanel;
	private javax.swing.JScrollPane portfolioTableScrollPane;

	private JTable portfolioTable;
	private JTable summaryTable;

	private DefaultTableModel portfolioTableModel;
	private DefaultTableModel summaryTableModel;

	private Object[][] portfolioData;
	private Object[][] summaryData;


	public PortfolioFrame(TradingGUI parentApplet) {
		this.parentApplet = parentApplet;
		this.userID = parentApplet.getLoggedUser().getUserID();

		setTitle("Portfolio");
		setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
		setDefaultCloseOperation(HIDE_ON_CLOSE); // this will allow the listener to dispose of this
		setResizable(true);

		initComponents();
		setLocationRelativeTo(parentApplet);

		SwingUtilities.invokeLater( new Runnable() {
			@Override
			public void run() {
				retrievePortfolio();
			}
		});
	}
	
	private void initComponents() {

		DefaultTableCellRenderer cellRenderer = new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if (value != null) {
					DecimalFormat df = new PriceFormat(); //DecimalFormat("0.0000;-0.0000");
					
					// add percent for the last column
					if( table.getColumnName(column).equals(tableColumns[8]) ) ((JLabel)c).setText(df.format((Float)value) + "%");
					else ((JLabel)c).setText(df.format((Float)value));
				}
				return c;
			}
		};

		portfolioTableScrollPane = new javax.swing.JScrollPane();
		footerPanel = new javax.swing.JPanel();
		loadingPanel = new javax.swing.JPanel();
		javax.swing.JPanel buttonPanel = new javax.swing.JPanel();
		SpringLayout layout = new SpringLayout();

		javax.swing.JButton refreshButton = new javax.swing.JButton();
		javax.swing.JButton saveButton = new javax.swing.JButton();
		javax.swing.JButton closeButton = new javax.swing.JButton();

		portfolioTableModel = new DefaultTableModel(portfolioData, tableColumns) {
			private static final long serialVersionUID = 1L;
			@Override
			public boolean isCellEditable(int x, int y) {
				return false;
			}
			@Override
			public Class<?> getColumnClass(int column) {
				Class<?> c = null;
				switch(column) {
				case 0: c = String.class; break;
				case 1: c = Date.class; break;
				case 2: c = Integer.class; break;
				case 3: // fall-through
				case 4: // fall-through
				case 5: // fall-through
				case 6: // fall-through
				case 7: // fall-through
				case 8: c = Float.class; break; // this uses the cell renderer
				default: c = String.class;
				}
				return c;
			}
		};

		portfolioTable = new JTable(portfolioTableModel) {
			private static final long serialVersionUID = 1L;
			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
				Component c = super.prepareRenderer(renderer, row, column);
				if (!c.getBackground().equals(getSelectionBackground())) {
					int realrow = getRowSorter().convertRowIndexToModel(row);
					Float changed = (Float)(getModel().getValueAt(realrow, 7));
					c.setBackground( changed != null && changed >= 0 ? new Color(223,255,146) : new Color(255,205,216) );
				}
				return c;
			}
		};

		summaryTableModel = new DefaultTableModel(summaryData, tableColumns) {
			private static final long serialVersionUID = 1L;
			@Override
			public boolean isCellEditable(int x, int y) {
				return false;
			}
			@Override
			public Class<?> getColumnClass(int column) {
				Class<?> c = null;
				switch(column) {
				case 0: c = String.class; break;
				case 1: c = Date.class; break;
				case 2: c = Integer.class; break;
				case 3: // fall-through
				case 4: // fall-through
				case 5: // fall-through
				case 6: // fall-through
				case 7: // fall-through
				case 8: c = Float.class; break; // this uses the cell renderer
				default: c = String.class;
				}
				return c;
			}
		};

		summaryTable = new JTable(summaryTableModel) {
			private static final long serialVersionUID = 1L;
			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
				Component c = super.prepareRenderer(renderer, row, column);
				if (!c.getBackground().equals(getSelectionBackground())) {
					Float changed = (Float)(getModel().getValueAt(0, 7));
					c.setBackground( changed != null && changed >= 0 ? new Color(223,255,146) : new Color(255,205,216) );
				}
				return c;
			}
		};
		
		summaryTable.setEnabled(false);
		summaryTable.setDefaultRenderer(Float.class, cellRenderer);
		summaryTable.setColumnModel(portfolioTable.getColumnModel());
		summaryTable.setTableHeader(portfolioTable.getTableHeader());
		portfolioTable.getTableHeader().setTable(portfolioTable);

		portfolioTable.setDefaultRenderer(Float.class, cellRenderer);
		portfolioTable.setAutoCreateRowSorter(true);
		portfolioTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_ALL_COLUMNS);
		portfolioTable.getTableHeader().setReorderingAllowed(false);

		JPanel tableHolder = new JPanel();
		BoxLayout b = new BoxLayout(tableHolder, BoxLayout.PAGE_AXIS);
		tableHolder.setLayout(b);

		tableHolder.add(portfolioTable);
		tableHolder.add(summaryTable);

		portfolioTableScrollPane.setColumnHeaderView(portfolioTable.getTableHeader());
		portfolioTableScrollPane.setViewportView(tableHolder);
		portfolioTableScrollPane.setPreferredSize(new Dimension(600, 400));

		refreshButton.setText("Refresh"); refreshButton.setMnemonic('f');
		refreshButton.addActionListener(new java.awt.event.ActionListener()
                {
                    @Override
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        loadingPanel.setVisible(true);
                        SwingUtilities.invokeLater(new Runnable()
                        {
                            @Override
                            public void run() {
                                loadingPanel.removeAll();
                                loadingPanel.add(new JLabel("Loading... Please wait..."));
                                retrievePortfolio();
                            }
                        });
                    }
		});

		saveButton.setText("Export"); saveButton.setMnemonic('e');
		saveButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveButtonActionPerformed(evt);
			}
		});

		closeButton.setText("Close"); closeButton.setMnemonic('c');
		closeButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				setVisible(false);
			}
		});

		buttonPanel.add(refreshButton);
		buttonPanel.add(saveButton);
		buttonPanel.add(closeButton);
                loadingPanel.removeAll();
		loadingPanel.add(new JLabel("Loading... Please wait..."));
                        
		footerPanel.setLayout(layout);
		footerPanel.setPreferredSize(new java.awt.Dimension(800, 35));
		footerPanel.add(loadingPanel, BorderLayout.WEST);
		footerPanel.add(buttonPanel, BorderLayout.CENTER);

		layout.putConstraint(SpringLayout.NORTH, loadingPanel, 5, SpringLayout.NORTH, footerPanel);
		layout.putConstraint(SpringLayout.EAST, buttonPanel, 0, SpringLayout.EAST, footerPanel);

		getContentPane().add(portfolioTableScrollPane, java.awt.BorderLayout.CENTER);
		getContentPane().add(footerPanel, java.awt.BorderLayout.PAGE_END);

		pack();
	}

	public void retrievePortfolio() {

		//make the portfolio sender
		GUIPortfolioListSenderWithReply portofolioSender;
		try { 
			portofolioSender = new GUIPortfolioListSenderWithReply(TradingGUI.IMQ_ADDRESS,
					Enums.QueueName.PORTFOLIO_LIST_REQUEST_QUEUE.toString(),
					Enums.QueueName.POTFOLIO_LIST_REPLY_QUEUE.toString(),
					userID+"-portfolio",
					parentApplet);

			//make the control
			Control portofolioControl = new Control(userID,
					Enums.ControlType.PORTFOLIO_LIST_REQUEST,
					null);

			//get portfolio list
			PortfolioList portfolioList = portofolioSender.sendControlGetReply(portofolioControl);

			//close the connection
			portofolioSender.close();

			//get portfolio queue
			ArrayDeque<Portfolio> portfolioArrayDequeue = portfolioList.getPortfolioList();

			//transform the ArrayDequeue into an ArrayList
			ArrayList<Portfolio> portfolioArrayList = new ArrayList<Portfolio>();
			while (!portfolioArrayDequeue.isEmpty()) {
				portfolioArrayList.add(portfolioArrayDequeue.pollFirst());
			}

			// get the updated instrument prices before displaying
			parentApplet.requestInstrumentList();

			// display portfolio
			populateTreeAndTable(portfolioArrayList);

		} catch (Exception e) {
			e.printStackTrace();
			parentApplet.showPopupError(e.getMessage());
		}
	}
        
	private void populateTreeAndTable(ArrayList<Portfolio> portfolioArrayList) {
            int portfolioSize = portfolioArrayList.size();
            int portfolioNumCols = tableColumns.length;
            Object[][] portfolioTempData = new Object[portfolioSize][portfolioNumCols];
            if( summaryData == null ) summaryData = new Object[1][tableColumns.length];

            float sumNetChanged = 0;
            float sumCrtValue = 0;
            float sumAquisitionValue = 0;
            float currPrice = 0;

           if (portfolioSize == 0)
            {
                loadingPanel.removeAll();
                loadingPanel.add(new JLabel("There are no portfolio entries"));
            } else {
                for (int i = 0; i < portfolioSize; i++) 
                {
                    Portfolio portfolioSymbol = portfolioArrayList.get(i);
                    portfolioTempData[i][0] = portfolioSymbol.getSymbol();
                    portfolioTempData[i][1] = portfolioSymbol.getLastUpdate();
                    portfolioTempData[i][2] = portfolioSymbol.getTotalQuant();
                    portfolioTempData[i][3] = (float) portfolioSymbol.getAvgPrice();

                    //set the current price
                    InstrumentList ilist = parentApplet.getInstrumentList();
                    if(ilist != null) {
                            Instrument inst = ilist.getInstrument(portfolioSymbol.getSymbol());
                            if (inst != null)
                                currPrice = inst.getCurrentPrice();
                            else
                                currPrice = (float) portfolioSymbol.getAvgPrice();
                    } else 
                        // currPrice = 0.000001F;
                        currPrice = (float) portfolioSymbol.getAvgPrice();
                    
                    portfolioTempData[i][4] = (float) currPrice;

                    //set the acquisition value
                    float aqValue = portfolioSymbol.getTotalQuant() * portfolioSymbol.getAvgPrice();
                    portfolioTempData[i][5] = (float) aqValue;
                    sumAquisitionValue += aqValue;

                    //set the current value = current price * acquisition quantity
                    float currValue = currPrice * portfolioSymbol.getTotalQuant();
                    portfolioTempData[i][6] = (float) currValue;
                    sumCrtValue += currValue;

                    //set the net change
                    portfolioTempData[i][7] = (float) (currValue - aqValue);
                    sumNetChanged += currValue - aqValue;

                    //set the % change
                    if( aqValue == 0 ) aqValue = 0.000001F;
                    float tmp = currValue / aqValue;

                    if (tmp >= 1) {
                        portfolioTempData[i][8] = (float) (100 * (tmp - 1));
                    } else if (tmp < 1) {
                        if (tmp == 0)
                            portfolioTempData[i][8] = 0F;
                        else    
                            portfolioTempData[i][8] = (float) (-100 * (1 - tmp));
                    }
                    
                    loadingPanel.setVisible(false);
                }
           }
		
            /* complete the summary line */
            summaryData[0][0] = "TOTAL";
            summaryData[0][5] = (float) sumAquisitionValue;
            summaryData[0][6] = (float) sumCrtValue;
            summaryData[0][7] = (float) sumNetChanged;

            if( sumAquisitionValue == 0 ) sumAquisitionValue = 0.000001F;
            float percentChanged = sumCrtValue / sumAquisitionValue;

            if (percentChanged >= 1) {
                summaryData[0][8] = (float) (100 * (percentChanged - 1));
            } else if (percentChanged < 1) {
                if (percentChanged == 0)
                    summaryData[0][8] = 0F;
                else
                    summaryData[0][8] = (float) (-100 * (1 - percentChanged));
            }

            portfolioData = portfolioTempData; //.clone();

            portfolioTableModel.setDataVector(portfolioData, tableColumns);
            summaryTableModel.setDataVector(summaryData, tableColumns);
	}


	private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {
		CSVWriter csv = null;
		String name = UseFileDialog.saveFile(this, "Export portfolio...", ".", "portfolio.csv");
		if( name == null || name.length() == 0 ) return;

		if( !name.toLowerCase().endsWith(".csv") && !name.toLowerCase().endsWith(".txt") && !name.toLowerCase().endsWith(".tsv") ) {
			name = name + ".csv";
		}

		try {
			csv = new CSVWriter(new PrintWriter(new BufferedWriter(new FileWriter(name))), 1, ',', '\"', '#', true);

			// header
			for (int i = 0; i < portfolioTable.getColumnCount(); i++) {
				csv.put(portfolioTable.getColumnName(i));
			}
			csv.nl();

			// data
			for (int j = 0; j < portfolioTable.getRowCount() - 1; j++) {
				for (int i = 0; i < portfolioTable.getColumnCount(); i++) {
					csv.put(portfolioTable.getValueAt(j, i).toString());
				}
				csv.nl();
			}

			// summary
			for (int i = 0; i < portfolioTable.getColumnCount(); i++) {
				if( summaryTable.getValueAt(0, i) == null ) csv.put("");
				else csv.put(summaryTable.getValueAt(0, i).toString());
			}

		} catch (IOException ex) {
			parentApplet.showPopupError(ex.getMessage());

		} finally {
			try { csv.close(); }
			catch(Exception e) { /* do nothing */ }
		}
	}
}
