package asets.gui.frames;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.ArrayDeque;
import java.util.Collection;
import java.util.Iterator;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.swing.JSpinner;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import asets.api.Control;
import asets.api.Enums;
import asets.api.Order;
import asets.api.OrderList;
import asets.api.Portfolio;
import asets.api.PortfolioList;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUIPortfolioListSenderWithReply;

public class SellFrame extends AbstractOrdersFrame {

    private static final long serialVersionUID = 1L;
    private GUIPortfolioListSenderWithReply portfolioListSender;
    private OrderList orderList;
    private PortfolioList portfolioList;
    private SortedMap<String,Portfolio> sortedMapPortfolio;
    private long maxQuant = 1;


    public SellFrame(TradingGUI parentApplet) {
        this.parentApplet = parentApplet;
        this.instrumentList = parentApplet.getInstrumentList();
        this.side = Enums.Side.SELL;

        setTitle("New order - SELL");
        setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
        setDefaultCloseOperation(HIDE_ON_CLOSE); // this will allow the listener to dispose of this
        setResizable(false);

        initComponents();
        setLocationRelativeTo(parentApplet);

        SwingUtilities.invokeLater( new Runnable() {
            
            @Override
            public void run() {
                resetForm();
            }
        });
    }

    @Override
    public void keyTypedHandler(KeyEvent evt) {
        super.keyTypedHandler(evt);
        if((Long)((JSpinner.DefaultEditor)quantitySpinner.getEditor()).getTextField().getValue() > maxQuant ||
            (Long)((JSpinner.DefaultEditor)quantitySpinner.getEditor()).getTextField().getValue() < 1) {
            statusBar.setLabelText("Invalid quantity");
        } else 
            statusBar.setLabelText("");
    }


    private void resetForm() {
        retrieveSymbols();
        retrieveOrders();

        if (populateComboBoxSymbol()) {

            Portfolio portfolio = (Portfolio)sortedMapPortfolio.get(symbolComboBox.getSelectedItem().toString());
            maxQuant = portfolio.getTotalQuant();
            quantitySpinner.setValue(maxQuant);

            symbolComboBox.setSelectedIndex(0);
            orderTypeComboBox.setSelectedIndex(0);
            priceSpinner.setEnabled(true);
            sendButton.setEnabled(true);
            
            orderTypeComboBox.addItemListener(new ItemListener() {

                public void itemStateChanged(ItemEvent e) {
                    if (e.getItem().toString().equals("Market Price")) {
                        priceSpinner.setEnabled(false);
                    } else {
                        priceSpinner.setEnabled(true);
                    }
                }
            });
        } else {
            parentApplet.showPopupError("You don't have anything to sell.");
        }
    }


    /**
     * Retrieves symbols that are included in user's portfolio
     */
    private void retrieveSymbols() {
        try {
            sortedMapPortfolio = new TreeMap<String,Portfolio>();
            portfolioListSender = new GUIPortfolioListSenderWithReply(TradingGUI.IMQ_ADDRESS,
                    Enums.QueueName.PORTFOLIO_LIST_REQUEST_QUEUE.toString(),
                    Enums.QueueName.POTFOLIO_LIST_REPLY_QUEUE.toString(), 
                    parentApplet.getLoggedUser().getUserID()+"-portfolio_sell", parentApplet);
            portfolioList = portfolioListSender.sendControlGetReply(new Control(parentApplet.getLoggedUser().getUserID(),
                    Enums.ControlType.PORTFOLIO_LIST_REQUEST, ""));
            ArrayDeque<Portfolio> portofolioListArrayDeque = portfolioList.getPortfolioList();
            for (Iterator<Portfolio> iterPortofolio = portofolioListArrayDeque.iterator(); iterPortofolio.hasNext();  ) {
                Portfolio temp = iterPortofolio.next();
                sortedMapPortfolio.put(temp.getSymbol(), temp);
            }
        } catch(Exception ex) {
                parentApplet.showPopupError(ex.getMessage());
        } finally {
            try {
                if(portfolioListSender != null)
                    portfolioListSender.close();
            } catch(Exception ex) {
                parentApplet.showPopupError(ex.getMessage());
            }
        }
    }


    /**
     * Retrieves currently pending orders to evaluate remaining quantity for each symbol
     */
    private void retrieveOrders() {	
        orderList = parentApplet.getOrderList();
        ArrayDeque<Order> orderListArrayDeque = orderList.getOrderList();

        for (Iterator<Order> iterator = orderListArrayDeque.iterator(); iterator.hasNext(); ) {
            Order order = iterator.next();
            if (order.getOrderSide().equals(Enums.Side.SELL)) {
                Portfolio portfolio = sortedMapPortfolio.get(order.getSymbol());
                if (portfolio != null) {
                    if (order.getMarketStatus() == Enums.MarketStatus.ON_MARKET ||
                        order.getMarketStatus() == Enums.MarketStatus.PENDING ||
                        order.getMarketStatus() == Enums.MarketStatus.PARTIALLY_EXECUTED) {
                        // Need to take into account, and subtract from the available quantity for
                        // SELL in the GUI, only the quantity yet uncertain, still not executed,
                        // and not reflected in the client's porfolio (PMS and DB). 
                        portfolio.setTotalQuant(portfolio.getTotalQuant() - order.getRemainedQuant());
                    }
                    sortedMapPortfolio.put(order.getSymbol(), portfolio);
                }
            }
        }
    }


    /**
     * Populates the select-box with previously retrieved symbols
     */
    private Boolean populateComboBoxSymbol() {
        symbolComboBox.removeAllItems();
        Collection<Portfolio> portfolioListCollection = sortedMapPortfolio.values();
        for(Portfolio i:portfolioListCollection) {
            if (i.getTotalQuant() > 0)
                symbolComboBox.addItem(i.getSymbol());
        }
        if (portfolioListCollection.size() > 0) 
            return true;
        else 
            return false;
    }


    protected void initComponents() {
        initCommonComponent();

        sendButton.setEnabled(false);
        
        symbolComboBox.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == 1) {
                    Portfolio portfolio = (Portfolio)sortedMapPortfolio.get(e.getItem().toString());
                    maxQuant = portfolio.getTotalQuant();
                    //quantitySpinner.setModel(new javax.swing.SpinnerNumberModel(Long.valueOf(1), Long.valueOf(1), Long.valueOf(maxQuant), Long.valueOf(1)));
                    quantitySpinner.setValue(maxQuant);
                }
            }
        });

        quantitySpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                if ((Long)quantitySpinner.getValue() > maxQuant ||
                    (Long)quantitySpinner.getValue() < 1) {
                    statusBar.setLabelText("Invalid quantity");
                } else
                    statusBar.setLabelText("");
            }
        });

        pack();
    }
}
