/**
 * Title:       GUI Frame class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements the functionality for a component in GUI
 */
package asets.gui.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import javax.jms.JMSException;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.RowFilter;
import javax.swing.RowFilter.ComparisonType;
import javax.swing.SpringLayout;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableRowSorter;

import asets.api.Control;
import asets.api.Enums;
import asets.api.Trade;
import asets.api.TradeList;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUITradeListSenderWithReply;
import asets.gui.core.CSVWriter;
import asets.gui.core.PriceFormat;
import asets.gui.core.UseFileDialog;

import com.toedter.calendar.JDateChooser;
import com.toedter.calendar.JSpinnerDateEditor;


/**
 *
 * @author Alin
 */
public class TradesFrame extends JFrame {

	private static final long serialVersionUID = 1L;
	protected static final String[] tableColumns = new String[]{ "Date", "Symbol", "Side", "Quantity", "Price", "Value", "Trade ID", "Order ID", "Execution ID" };

	private TradingGUI parentApplet;
	private String userID;

	private javax.swing.JPanel loadingPanel;
	private javax.swing.JPanel footerPanel;
	private javax.swing.JPanel filterPanel;
	private javax.swing.JScrollPane tradesTableScrollPane;
	private javax.swing.JTable tradesTable;
	
	private JDateChooser fromDate;
	private JDateChooser toDate;

	private DefaultTableModel tradesTableModel;
	private Object[][] tradesData;
	
	
	public TradesFrame(TradingGUI parentApplet)  {
		this.parentApplet = parentApplet;
		this.userID = parentApplet.getLoggedUser().getUserID();

		setTitle("Trades");
		setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
		setDefaultCloseOperation(HIDE_ON_CLOSE); // this will allow the listener to dispose of this
		setResizable(true);

		initComponents();
		setLocationRelativeTo(parentApplet);

		SwingUtilities.invokeLater( new Runnable() {
			@Override
			public void run() {
				loadTrades();
				calculateDates();
			}
		});

	}


	private void initComponents() {

		DefaultTableCellRenderer cellRenderer = new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if (value != null) {
					DecimalFormat df = new PriceFormat(); //DecimalFormat("0.0000;-0.0000");
					((JLabel)c).setText(df.format((Float)value));
				}
				return c;
			}
		};

		tradesTableScrollPane = new javax.swing.JScrollPane();
		footerPanel = new javax.swing.JPanel();
		loadingPanel = new javax.swing.JPanel();
		filterPanel = new javax.swing.JPanel(new java.awt.FlowLayout(FlowLayout.LEFT));
		javax.swing.JPanel buttonPanel = new javax.swing.JPanel();
		SpringLayout layout = new SpringLayout();

		javax.swing.JButton refreshButton = new javax.swing.JButton();
		javax.swing.JButton saveButton = new javax.swing.JButton();
		javax.swing.JButton closeButton = new javax.swing.JButton();
		javax.swing.JButton showAllButton = new javax.swing.JButton();
		javax.swing.JButton filterButton = new javax.swing.JButton();

		tradesTableModel = new javax.swing.table.DefaultTableModel( tradesData, tableColumns ) {
			private static final long serialVersionUID = 1L;
			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return false;
			}
			@Override
			public Class<?> getColumnClass(int column) {
				Class<?> c = null;
				switch(column) {
				case 0: c = Date.class; break;
				case 1: // fall-through
				case 2: c = String.class; break;
				case 3: c = Integer.class; break;
				case 4: // fall-through
				case 5: c = Float.class; break;
				case 6: // fall-through
				case 7: // fall-through
				case 8: c = String.class; break;
				default: c = String.class;
				}
				return c;
			}
		};

		tradesTable = new javax.swing.JTable(tradesTableModel) {
			private static final long serialVersionUID = 1L;
			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
				Component c = super.prepareRenderer(renderer, row, column);
				if (!c.getBackground().equals(getSelectionBackground())) {
					int realrow = getRowSorter().convertRowIndexToModel(row);
					String side = (String)(getModel().getValueAt(realrow, 2));
					c.setBackground( side != null && side.equalsIgnoreCase("buy") ? new Color(223,255,146) : new Color(255,205,216) );
				}
				return c;
			}
		};

		tradesTable.getTableHeader().setReorderingAllowed(true);
		tradesTable.setDefaultRenderer(Float.class, cellRenderer);
		tradesTable.setAutoCreateRowSorter(true);
		tradesTableScrollPane.setViewportView(tradesTable);

		refreshButton.setText("Refresh"); refreshButton.setMnemonic('f');
		refreshButton.addActionListener(new java.awt.event.ActionListener()
                {
                    @Override
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        filterPanel.setVisible(false);
                        loadingPanel.setVisible(true);
                        SwingUtilities.invokeLater(new Runnable() 
                        {
                            @Override
                            public void run() {
                                loadingPanel.removeAll();
                                loadingPanel.add(new JLabel("Loading... Please wait..."));
                                loadTrades();
                            }
                        });
                    }
		});

		saveButton.setText("Export"); saveButton.setMnemonic('e');
		saveButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveButtonActionPerformed(evt);
			}
		});

		closeButton.setText("Close"); closeButton.setMnemonic('c');
		closeButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				setVisible(false);
			}
		});
		
		filterButton.setText("Filter"); filterButton.setMnemonic('f');
		filterButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				applyFilter();
			}
		});

		showAllButton.setText("Show all"); showAllButton.setMnemonic('w');
		showAllButton.addActionListener(new java.awt.event.ActionListener() {
			@SuppressWarnings({ "unchecked", "rawtypes" })
			@Override
			public void actionPerformed(ActionEvent e) {
				((TableRowSorter) tradesTable.getRowSorter()).setRowFilter(null);
			}
		});
		
		
		// create date pickers
		fromDate = new JDateChooser(null, null, null,new JSpinnerDateEditor());
		toDate = new JDateChooser(null, null, null,new JSpinnerDateEditor());
		toDate.addPropertyChangeListener("date", new PropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				try {
					if( fromDate.getDate() == null ) return;
					Calendar a = Calendar.getInstance();
					a.setTime((Date) evt.getNewValue());
					a.add(Calendar.DAY_OF_YEAR, -1);
					
					if( fromDate.getDate().after( (Date) evt.getNewValue()) || fromDate.getDate().equals( (Date) evt.getNewValue()) ) {
						fromDate.setDate(a.getTime());
					}
					fromDate.setMaxSelectableDate(a.getTime());
				} catch(Exception e) {
					e.printStackTrace();
				}
			}
		});
		
		Calendar tomorrow = Calendar.getInstance();
		tomorrow.add(Calendar.DAY_OF_YEAR, 1);
		
		toDate.setMaxSelectableDate(tomorrow.getTime());
		
		filterPanel.add(new JLabel("From: "));
		filterPanel.add(fromDate);
		filterPanel.add(new JLabel(" To:"));
		filterPanel.add(toDate);
		filterPanel.add(filterButton);
		filterPanel.add(showAllButton);
		filterPanel.setVisible(false);
		filterPanel.setPreferredSize(new java.awt.Dimension(450, 35));

		buttonPanel.add(refreshButton);
		buttonPanel.add(saveButton);
		buttonPanel.add(closeButton);
                loadingPanel.removeAll();
		loadingPanel.add(new JLabel("Loading... Please wait..."));

		footerPanel.setLayout(layout);
		footerPanel.setPreferredSize(new java.awt.Dimension(800, 35));
		footerPanel.add(loadingPanel, BorderLayout.WEST);
		footerPanel.add(filterPanel, BorderLayout.CENTER);
		footerPanel.add(buttonPanel, BorderLayout.EAST);
		
		layout.putConstraint(SpringLayout.NORTH, loadingPanel, 5, SpringLayout.NORTH, footerPanel);
		layout.putConstraint(SpringLayout.EAST, buttonPanel, 0, SpringLayout.EAST, footerPanel);
		layout.putConstraint(SpringLayout.WEST, filterPanel, 0, SpringLayout.WEST, footerPanel);

		getContentPane().add(tradesTableScrollPane, java.awt.BorderLayout.CENTER);
		getContentPane().add(footerPanel, java.awt.BorderLayout.PAGE_END);

		setMinimumSize(new java.awt.Dimension(700, 300));
                
		pack();
	}

	public void loadTrades() {
            GUITradeListSenderWithReply tradesSender = null;
            
            try
            {
                tradesSender = new GUITradeListSenderWithReply(TradingGUI.IMQ_ADDRESS,
                        Enums.QueueName.TRADE_LIST_REQUEST_QUEUE.toString(),
                        Enums.QueueName.TRADE_LIST_REPLY_QUEUE.toString(), userID, parentApplet);

                //creating the control
                Control tradeControl = new Control(userID,
                        Enums.ControlType.TRADE_LIST_REQUEST, "");

                //get the trade list
                TradeList tradeSymbols = new TradeList();
                tradeSymbols = tradesSender.sendControlGetReply(tradeControl);
                
                if (tradesSender != null)
                    tradesSender.close();

                if( tradeSymbols == null ) return;
                ArrayDeque<Trade> tradeList = tradeSymbols.getTradeList();

                tradesData = new Object[tradeList.size()][tableColumns.length];

                if (tradeList.isEmpty())
                {
                    loadingPanel.removeAll();
                    loadingPanel.add(new JLabel("There are no trade entries"));
                } else {
                    int i = 0;
                    while(tradeList.size() > 0)
                    {
                        float tradePrice;
                        long tradeQuantity;
                        Trade tmpTrade = tradeList.pollFirst();

                        if( tmpTrade == null ) break;
                        tradePrice = tmpTrade.getTradePrice();
                        tradeQuantity = tmpTrade.getTradeQuant();

                        tradesData[i][0] = tmpTrade.getTradeTime();
                        tradesData[i][1] = tmpTrade.getSymbol();
                        tradesData[i][2] = tmpTrade.getTradeSide().toString();
                        tradesData[i][3] = tradeQuantity;
                        tradesData[i][4] = tradePrice;
                        tradesData[i][5] = (float) (tradePrice * tradeQuantity);
                        tradesData[i][6] = tmpTrade.getTradeID();
                        tradesData[i][7] = tmpTrade.getOrderID();
                        tradesData[i][8] = tmpTrade.getExecID();
                        i++;

                        tradesTableModel.setDataVector(tradesData, tableColumns);

                        loadingPanel.setVisible(false);
                        filterPanel.setVisible(true);
                    }
                }
            } catch (Exception e) {
                if (tradesSender != null) {
                    try {
                        tradesSender.close();
                    } catch (JMSException ex1) {
                      parentApplet.showPopupError(ex1.getMessage());            
                    }
                }
                
                loadingPanel.setVisible(false);
                e.printStackTrace();
                parentApplet.showPopupError(e.getMessage());
            } finally {
                if (tradesSender != null) {
                    try {
                        tradesSender.close();
                    } catch(Exception ex2) {
                        parentApplet.showPopupError(ex2.getMessage());
                    }
                }
            }     
	}
	
	private void calculateDates() {
		Calendar cal = Calendar.getInstance();
		
		if(tradesTableModel.getRowCount() > 0) {
			Date last = (Date) tradesTableModel.getValueAt(tradesTableModel.getRowCount()-1, 0);
			if( last != null ) cal.setTime(last);
		}
		
		// calculate the exact start of the day
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		fromDate.setDate( cal.getTime() );
		
		// calculate the next day
		cal.add(Calendar.DAY_OF_YEAR, 1);
		toDate.setDate( cal.getTime() );
		
		applyFilter();
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void applyFilter() {
		RowFilter<DefaultTableModel, Date> rowFilter = null;
		try {
			ArrayList<RowFilter<DefaultTableModel, Date>> list = new ArrayList<RowFilter<DefaultTableModel,Date>>(2);
			
			rowFilter = RowFilter.dateFilter(ComparisonType.AFTER, fromDate.getDate());
			list.add( rowFilter );
			
			rowFilter = RowFilter.dateFilter(ComparisonType.BEFORE, toDate.getDate());
			list.add( rowFilter );
			
			((TableRowSorter) tradesTable.getRowSorter()).setRowFilter(RowFilter.andFilter(list));
		} catch( Exception ex ) {
			ex.printStackTrace();
		}
	}
	
	
	private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {
		CSVWriter csv = null;
		String name = UseFileDialog.saveFile(this, "Export trades...", ".", "trades.csv");
		if( name == null || name.length() == 0 ) return;

		if( !name.toLowerCase().endsWith(".csv") && !name.toLowerCase().endsWith(".txt") && !name.toLowerCase().endsWith(".tsv") ) {
			name = name + ".csv";
		}

		try {
			csv = new CSVWriter(new PrintWriter(new BufferedWriter(new FileWriter(name))), 1, ',', '\"', '#', true);

			// header
			for (int i = 0; i < tradesTable.getColumnCount(); i++) {
				csv.put(tradesTable.getColumnName(i));
			}
			csv.nl();

			// data
			for (int j = 0; j < tradesTable.getRowCount() - 1; j++) {
				for (int i = 0; i < tradesTable.getColumnCount(); i++) {
					csv.put(tradesTable.getValueAt(j, i).toString());
				}
				csv.nl();
			}

		} catch (IOException ex) {
			parentApplet.showPopupError(ex.getMessage());

		} finally {
			try { csv.close(); }
			catch(Exception e) { /* do nothing */ }
		}

	}

}
