/**
 * Title:       Orders Panel class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements the functionality for a Orders and Executions panel in GUI.
 * 				This panel is the main GUI windows for logged in users and shows the
 * 				entire GUI elements, including Order list, Execution list, etc
 */

package asets.gui.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayDeque;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableRowSorter;

import asets.api.Control;
import asets.api.Enums;
import asets.api.Execution;
import asets.api.ExecutionList;
import asets.api.ExecutionListSenderWithReply;
import asets.api.Order;
import asets.api.OrderList;
import asets.api.OrderListSenderWithReply;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUIOrderListSenderWithReply;
import asets.gui.core.PriceFormat;
import asets.gui.menus.OrdersPopupMenu;

public class OrdersPanel extends javax.swing.JPanel {

	private static final long serialVersionUID = 1L;
	private final String[] orderColumnNames = {"Order ID", "Order Time", "Symbol", "Side","Type","Price","Quantity","Remaining","Order Status","Market Status","Last Updated","Client Order ID"};
	private final String[] execColumnNames = {"Execution ID", "Execution Time", "Order ID", "Symbol","Side","Price","Quantity"};

	// what to print in the price column when MP
	private final String MP_SYMBOL_BUY = "MP"; 
	private final String MP_SYMBOL_SELL = "MP "; // slightly different than the above to allow sorting
	
	private javax.swing.JPanel executionsPanel;
	private javax.swing.JLabel executionsLabel;
	private javax.swing.JTable executionsTable;
	private javax.swing.JPanel ordersPanel;
	private javax.swing.JLabel ordersLabel;
	private javax.swing.JTable ordersTable;
	private javax.swing.JScrollPane topScrollPane;
	private javax.swing.JScrollPane bottomScrollPane;
	private javax.swing.JSplitPane mainSplitPane;

	DefaultTableModel ordersTableModel, executionsTableModel;
	OrdersPopupMenu ordersPopupMenu;
	TradingGUI parentApplet;
	OrderList orderList;
	HashMap<String, Order> orderListHashMap;

	public OrdersPanel(TradingGUI parentApplet) {
		this.parentApplet = parentApplet;
		orderListHashMap = new HashMap<String, Order>();

		String userID = parentApplet.getLoggedUser().getUserID();
		setBounds(0, 0, parentApplet.getWidth(), parentApplet.getHeight());
		setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.PAGE_AXIS));

		initComponents();


		// Orders and Executions retrieval
		System.out.println("Loading orders...");
		OrderListSenderWithReply orderListSender = null;

		try {
			Control control = new Control(userID, Enums.ControlType.CLIENT_ORDER_LIST_REQUEST, null);
			orderListSender = new GUIOrderListSenderWithReply(TradingGUI.IMQ_ADDRESS,
					Enums.QueueName.CLIENT_ORDER_LIST_REQUEST_QUEUE.toString(),
					Enums.QueueName.CLIENT_ORDER_LIST_REPLY_QUEUE.toString(),
					userID);
			orderList = orderListSender.sendControlGetReply(control);
                        System.out.println(orderList.getOrderList().size() + " orders received.");
                        
                        if (orderListSender != null)
                            orderListSender.close();
		}
		catch (Exception e){
			parentApplet.showPopupError(e.getMessage());
		}
		finally{
                    try{
                        if (orderListSender != null)
                            orderListSender.close();
                    }
                    catch (Exception ex){
                            parentApplet.showPopupError(ex.getMessage());
                    }
		}
		if (orderList!=null){
			System.out.println("Orders retrieved. Populating the table...");
			ArrayDeque<Order> orderListArrayDeque=orderList.getOrderList();
			for (Iterator<Order> iterator = orderListArrayDeque.iterator(); iterator.hasNext();  ) {
				Order order=iterator.next();
				addOrUpdateOrder(order);
			}
		}

		System.out.println("Loading executions...");
		ExecutionListSenderWithReply execListSender = null;
		ExecutionList execList = null;

		try {
			Control controlUser = new Control(userID, Enums.ControlType.CLIENT_EXECUTION_LIST_REQUEST, null);
			execListSender = new ExecutionListSenderWithReply(TradingGUI.IMQ_ADDRESS,
					Enums.QueueName.CLIENT_EXECUTION_LIST_REQUEST_QUEUE.toString(),
					Enums.QueueName.CLIENT_EXECUTION_LIST_REPLY_QUEUE.toString(),
					userID);
			execList = execListSender.sendControlGetReply(controlUser);
			System.out.println(execList.getExecutionList().size() + " executions received.");
                        
                        if (execListSender != null)
                            execListSender.close();                       
		}
		catch (Exception e){
			parentApplet.showPopupError(e.getMessage());
		}
		finally{
                    try{
                        if (execListSender != null)
                            execListSender.close();
                    }
                    catch (Exception ex){
                        parentApplet.showPopupError(ex.getMessage());
                    }
		}
		if (execList!=null){
			System.out.println("Executions retrieved. Populating the table..");
			Execution exec = execList.getExecution();
			int k=0;
			while ((exec!=null)&&(k<100)){
				addExecution(exec);
				exec = execList.getExecution();
				k++;
			}
		}
	}


	/*
	 * Returns current order list
	 * */
	public OrderList getOrderList() {
		orderList = new OrderList();
		for (Iterator<Order> i= orderListHashMap.values().iterator(); i.hasNext();) {
			orderList.addOrder(i.next());
		}
		return orderList;
	}

	/*
	 * Adds a new order
	 * */
	public synchronized void addOrder(Order order) {
		if (order.getOrderID() != null) orderListHashMap.put(order.getOrderID(), order);
		else orderListHashMap.put(order.getClientOrderID(), order);

		Object[] data = new Object[12];
		if (order.getOrderID()!= null) data[0] = order.getOrderID();
		else data[0] = "";
		if (order.getOrderTime()!= null) data[1] = order.getOrderTime();
		else data[1] = "";
		if (order.getSymbol() != null) data[2] = order.getSymbol();
		else data[2] = "";
		if (order.getOrderSide()!= null) data[3] = order.getOrderSide().toString();
		else data[3] = "";
		if (order.getOrderType() != null) data[4] = order.getOrderType().toString();
		else data[4] = "";
		if (order.getOrderType().equals(Enums.OrderType.LIMIT_PRICE)) data[5] = order.getOrderPrice();
		else {
			if( order.getOrderSide() == Enums.Side.BUY ) data[5] = MP_SYMBOL_BUY;
			else data[5] = MP_SYMBOL_SELL;
		}
		data[6] = order.getOrderQuant();
		data[7] = order.getRemainedQuant();
		if (order.getOrderStatus() != null) data[8] = order.getOrderStatus().toString();
		else data[8] = "";
		if (order.getMarketStatus() != null) data[9] = order.getMarketStatus().toString();
		else data[9] = "";
		if (order.getLastUpdate()!= null)data[10] = order.getLastUpdate();
		else data[10] = "";
		if (order.getClientOrderID() != null) data[11] = order.getClientOrderID();
		else data[11] = "";
		ordersTableModel.addRow(data);
		//orderListHashMap.offerLast(order);
		System.out.println("Order added in GUI. Order Status: "+order.getOrderStatus().toString()+"; Market status: "+order.getMarketStatus().toString());
	}

	public synchronized void updateOrder(Order order) {
		Boolean found = false;
		Boolean tooLate = false;
		Order oldOrder = null;
		for (int i=0; i<ordersTableModel.getRowCount() && !found; i++) {
			if (ordersTableModel.getValueAt(i, 0) != null && ordersTableModel.getValueAt(i, 0) != "") {
				if (ordersTableModel.getValueAt(i, 0).equals(order.getOrderID())) {
					found = true;
					oldOrder = orderListHashMap.get(order.getOrderID());
					if (oldOrder.getSeqNo() <= order.getSeqNo()) 
                                            orderListHashMap.remove(order.getOrderID());
					else 
                                            tooLate = true;
				}
			}
			else if (ordersTableModel.getValueAt(i, 11) != null && ordersTableModel.getValueAt(i, 11) != "") {
				if (ordersTableModel.getValueAt(i, 11).equals(order.getClientOrderID())) {
					found = true;
					ordersTableModel.setValueAt(order.getOrderID(), i, 0);
					oldOrder = orderListHashMap.get(order.getClientOrderID());
					if (oldOrder.getSeqNo() <= order.getSeqNo()) 
                                            orderListHashMap.remove(order.getClientOrderID());
					else 
                                            tooLate = true;
				}
			}
                        
			if (found == true && tooLate == false) {
				ordersTableModel.setValueAt(order.getOrderTime(), i, 1);
				ordersTableModel.setValueAt(order.getSymbol(), i, 2);
				ordersTableModel.setValueAt(order.getOrderSide().toString(), i, 3);
				ordersTableModel.setValueAt(order.getOrderType().toString(), i, 4);
				if (order.getOrderType().equals(Enums.OrderType.LIMIT_PRICE)) 
                                    ordersTableModel.setValueAt(order.getOrderPrice(), i, 5);
				else {
					if( order.getOrderSide() == Enums.Side.BUY ) 
                                            ordersTableModel.setValueAt(MP_SYMBOL_BUY, i, 5);
					else 
                                            ordersTableModel.setValueAt(MP_SYMBOL_SELL, i, 5);
				}
				ordersTableModel.setValueAt(order.getOrderQuant(), i, 6);
				ordersTableModel.setValueAt(order.getRemainedQuant(), i, 7);
				ordersTableModel.setValueAt(order.getOrderStatus().toString(), i, 8);
				ordersTableModel.setValueAt(order.getMarketStatus().toString(), i, 9);
				ordersTableModel.setValueAt(order.getLastUpdate(), i, 10);
				orderList.getOrderList().remove(oldOrder);
				orderList.getOrderList().add(order);
				orderListHashMap.put(order.getOrderID(), order);
			}
		}
		if (found) {
			if (!tooLate) 
                            System.out.println("Order updated in GUI. Order Status: "+order.getOrderStatus().toString()+". Market Status: "+order.getMarketStatus().toString()+"; SeqNo: "+order.getSeqNo());
			else 
                            System.out.println("Update received too late. Will be igonred.");
                } else 
                    System.out.println("Order to be updated not found.");
		//    Iterator<Order> orderIterator = orderListHashMap.iterator();
		//    while (orderIterator.hasNext()) {
		//        Order o = orderIterator.next();
		//        if (o.getOrderID().equals(order.getOrderID())) o = order;
		//    }
	}

	public synchronized void addOrUpdateOrder(Order order) {
		if (orderListHashMap.containsKey(order.getOrderID()) || orderListHashMap.containsKey(order.getClientOrderID()) ) {
			updateOrder(order);
		} else {
			addOrder(order);
		}
	}

	public synchronized void addOrIgnoreOrder(Order order) {
		if (orderListHashMap.containsKey(order.getOrderID()) || orderListHashMap.containsKey(order.getClientOrderID()) ) {
			System.out.println("Order already exists. Will be ignored.");
			return;
		} else {
			addOrder(order);
		}
	}

	public void addExecutionAndUpdateOrder(Execution exec) {
		addExecution(exec);
	}

	public synchronized void addExecution(Execution exec){
		Object[] data = new Object[7];
		if (exec.getExecID()!= null) data[0] = exec.getExecID();
		else data[0] = "";
		if (exec.getExecTime() != null) data[1] = exec.getExecTime();
		else data[1] = "";
		if (exec.getOrderID() != null) data[2] = exec.getOrderID();
		else data[2] = "";
		if (exec.getSymbol() != null) data[3] = exec.getSymbol();
		else data[3] = "";
		if (exec.getExecSide() != null) data[4] = exec.getExecSide().toString();
		else data[4] = "";
		data[5] = exec.getExecPrice();
		data[6] = exec.getExecQuant();

		executionsTableModel.addRow(data);
		System.out.println("Execution added in GUI");
		/*
        if (orderListHashMap.containsKey(exec.getOrderID())) {
            Order o = orderListHashMap.get(exec.getOrderID());
            // TODO : update order according to execution information
            // maybe set status to FULLY EXECUTED OR PARTIALLY EXECUTED, i don't know
            // maybe subtranct execution quantity
            long quantity = o.getOrderQuant();
            o.setOrderQuant(quantity - exec.getExecQuant());
            updateOrder(o);
            System.out.println("Order - " + exec.getOrderID() + " updated in GUI");
        } else {
            System.out.println("Order " + exec.getOrderID() + " for execution not found to update");
        }*/
	}

	public void selectExecutionsForOrder(String orderId) {
		int executionOrderIdIndex = 2;

		executionsTable.clearSelection();

		for (int index = 0; index < executionsTableModel.getRowCount(); index++) {
			if (executionsTableModel.getValueAt(index, executionOrderIdIndex) != null && executionsTableModel.getValueAt(index, executionOrderIdIndex) != "") {
				if (executionsTableModel.getValueAt(index, executionOrderIdIndex).equals(orderId)) {
					int rowToSelect = executionsTable.convertRowIndexToView(index);

					if (rowToSelect > -1) {
						executionsTable.addRowSelectionInterval(rowToSelect, rowToSelect);
					}
				}
			}
		}
	}

	private void initComponents() {

		mainSplitPane = new javax.swing.JSplitPane();

		ordersPanel = new javax.swing.JPanel();
		executionsPanel = new javax.swing.JPanel();

		ordersLabel = new javax.swing.JLabel();
		executionsLabel = new javax.swing.JLabel();

		topScrollPane = new javax.swing.JScrollPane();
		bottomScrollPane = new javax.swing.JScrollPane();


		ordersPanel.setLayout(new javax.swing.BoxLayout(ordersPanel, javax.swing.BoxLayout.PAGE_AXIS));
		ordersLabel.setFont(new java.awt.Font("Tahoma", 1, 11));
		ordersLabel.setText("Orders:");
		ordersLabel.setMaximumSize(new java.awt.Dimension(800, 14));
		ordersLabel.setPreferredSize(new java.awt.Dimension(800, 30));
		ordersPanel.add(ordersLabel);
		ordersPanel.add(topScrollPane);

		executionsPanel.setLayout(new javax.swing.BoxLayout(executionsPanel, javax.swing.BoxLayout.PAGE_AXIS));
		executionsLabel.setFont(new java.awt.Font("Tahoma", 1, 11));
		executionsLabel.setText("Executions:");
		executionsLabel.setMaximumSize(new java.awt.Dimension(800, 14));
		executionsLabel.setMinimumSize(new java.awt.Dimension(41, 14));
		executionsLabel.setPreferredSize(new java.awt.Dimension(800, 30));
		executionsPanel.add(executionsLabel);
		executionsPanel.add(bottomScrollPane);


		mainSplitPane.setDividerLocation(300);
		mainSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);

		mainSplitPane.setLeftComponent(ordersPanel);
		mainSplitPane.setRightComponent(executionsPanel);

		add(mainSplitPane);



		ordersTableModel = new DefaultTableModel(new Object[0][0], orderColumnNames) {
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int x, int y) {
				return false;
			}
		};

		ordersTable = new JTable(ordersTableModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column)
			{
				Component c = super.prepareRenderer(renderer, row, column);
				if (!c.getBackground().equals(getSelectionBackground()))
				{
					int realrow = getRowSorter().convertRowIndexToModel(row);
					String type = (String)getModel().getValueAt(realrow, 3);
					c.setBackground( type.equals(Enums.Side.BUY.toString()) ? new Color(223,255,146) : new Color(255,205,216) );
				}
				return c;
			}
		};
		ordersTable.removeColumn(ordersTable.getColumn("Client Order ID"));

		ordersTable.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if (value.getClass() == Date.class) {
					SimpleDateFormat df = new SimpleDateFormat("dd MMMM yyyy HH:mm:ss");
					((JLabel)c).setText(df.format((Date)value));
				}
				if (value.getClass() == Float.class) {
					DecimalFormat twoDForm = new PriceFormat();
					((JLabel)c).setText(twoDForm.format((Float)value));
				}

				return c;
			}

		});

		TableRowSorter<DefaultTableModel> ordersRowSorter = new TableRowSorter<DefaultTableModel>(ordersTableModel);
		ordersRowSorter.setComparator(1, new Comparator<Date>() {

			public int compare(Date o1, Date o2) {
				return o1.compareTo(o2);
			}

		});
		ordersRowSorter.setComparator(5, new Comparator<Object>() {

			/**
			 * Compares two prices in order to sort the orders table.
			 * This also treats a particular situation in which there are orders
			 * at Market Price, which do not have any price at all displayed in the cell. 
			 */
			public int compare(Object o1, Object o2) {
				if( o1.equals(MP_SYMBOL_BUY) ) return 1;
				else if( o1.equals(MP_SYMBOL_SELL) ) return -1;
				else if( o2.equals(MP_SYMBOL_BUY) ) return -1;
				else if( o2.equals(MP_SYMBOL_SELL) ) return 1;
				else return ((Float) o1).compareTo( (Float) o2);
			}

		});
		ordersRowSorter.setComparator(6,new Comparator<Long>() {

			public int compare(Long o1, Long o2) {
				return o1.compareTo(o2);
			}

		});
		ordersRowSorter.setComparator(7, new Comparator<Long>() {

			public int compare(Long o1, Long o2) {
				return o1.compareTo(o2);
			}

		});
		ordersRowSorter.setComparator(10,new Comparator<Date>() {

			public int compare(Date o1, Date o2) {
				return o1.compareTo(o2);
			}

		});
		// ordersTable.setAutoCreateRowSorter(true);
		ordersTable.setRowSorter(ordersRowSorter);

		topScrollPane.setViewportView(ordersTable);



		executionsTableModel = new DefaultTableModel(new Object[0][0],execColumnNames) {
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int x, int y) {
				return false;
			}

		};

		executionsTable = new JTable(executionsTableModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column)
			{
				Component c = super.prepareRenderer(renderer, row, column);
				if (!c.getBackground().equals(getSelectionBackground()))
				{
					int realrow = getRowSorter().convertRowIndexToModel(row);
					String type = (String)getModel().getValueAt(realrow, 4);
					c.setBackground(type.equals(Enums.Side.BUY.toString()) ? new Color(223,255,146) : new Color(255,205,216) );
				}
				return c;
			}
		};

		executionsTable.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if (value.getClass() == Date.class) {
					SimpleDateFormat df = new SimpleDateFormat("dd MMMM yyyy HH:mm:ss");
					((JLabel)c).setText(df.format((Date)value));
				}
				if (value.getClass() == Float.class) {
					DecimalFormat twoDForm = new PriceFormat();
					((JLabel)c).setText(twoDForm.format((Float)value));
				}

				return c;
			}

		});

		TableRowSorter<DefaultTableModel> executionsRowSorter = new TableRowSorter<DefaultTableModel>(executionsTableModel);
		executionsRowSorter.setComparator(1, new Comparator<Date>() {

			public int compare(Date o1, Date o2) {
				return o1.compareTo(o2);
			}

		});
		executionsRowSorter.setComparator(5, new Comparator<Float>() {

			public int compare(Float o1, Float o2) {
				return o1.compareTo(o2);
			}

		});
		executionsRowSorter.setComparator(6,new Comparator<Long>() {

			public int compare(Long o1, Long o2) {
				return o1.compareTo(o2);
			}

		});

		//executionsTable.setAutoCreateRowSorter(true);
		executionsTable.setRowSorter(executionsRowSorter);

		executionsTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		bottomScrollPane.setViewportView(executionsTable);



		ordersPopupMenu = new OrdersPopupMenu(parentApplet,"Options");

		ordersTable.setComponentPopupMenu(ordersPopupMenu);
		ordersTable.addMouseListener(new MouseAdapter() {

			@Override
			public void mousePressed(MouseEvent e) {
				maybeShowPopup(e);
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				maybeShowPopup(e);
			}

			private void maybeShowPopup(MouseEvent e) {
				int row = ordersTable.rowAtPoint(new Point(e.getX(), e.getY()));
				if ((Integer)row!=null) {
					ordersTable.setRowSelectionInterval(row, row);
				}
				int modelRow = ordersTable.convertRowIndexToModel(row);
				String orderID = ordersTableModel.getValueAt(modelRow, 0).toString();
				if (!orderID.isEmpty()) {

					selectExecutionsForOrder(orderID);

					if (orderListHashMap.containsKey(orderID)) {
						Order order = orderListHashMap.get(orderID);
						ordersPopupMenu.setClickedOrder(order);
						ordersPopupMenu.setAllowUpdate(
								(order.getMarketStatus() == Enums.MarketStatus.ON_MARKET ||
								order.getMarketStatus() == Enums.MarketStatus.PARTIALLY_EXECUTED)
								&&
								( order.getOrderStatus() == Enums.OrderStatus.ADDED ||
								order.getOrderStatus() == Enums.OrderStatus.ADD_FAILED ||
								order.getOrderStatus() == Enums.OrderStatus.UPDATED ||
								order.getOrderStatus() == Enums.OrderStatus.UPDATE_FAILED
										) );
						ordersPopupMenu.setAllowCancel(
								(order.getMarketStatus() == Enums.MarketStatus.ON_MARKET ||
								order.getMarketStatus() == Enums.MarketStatus.PARTIALLY_EXECUTED)
								&&
								( order.getOrderStatus() == Enums.OrderStatus.ADDED ||
								order.getOrderStatus() == Enums.OrderStatus.ADD_FAILED ||
								order.getOrderStatus() == Enums.OrderStatus.UPDATED ||
								order.getOrderStatus() == Enums.OrderStatus.UPDATE_FAILED
										)
								);
					}

				} else {
					ordersPopupMenu.setAllowCancel(false);
					ordersPopupMenu.setAllowUpdate(false);
				}

				// show pop-up
				if (e.isPopupTrigger()) {            
					ordersPopupMenu.show(e.getComponent(), e.getX(), e.getY());
				}
			}

		});

	}



}
