/**
 * Title:       Status Bar class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements the functionality for an extended Status Bar in GUI which
 * 				displays different kinds of information when the user is logged in.
 */

package asets.gui.panels;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.NumberFormat;

import javax.swing.Timer;

import asets.api.Control;
import asets.api.Enums;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUILogOutSenderNoReply;
/**
 * @author Adrian Bardan
 */
public class StatusBar extends javax.swing.JPanel implements ActionListener, MouseListener {
	private static final long serialVersionUID = -6871651302108536886L;
	
	private javax.swing.JLabel statusIcon;
	private javax.swing.JLabel statusLabel;
	private javax.swing.JLabel timerLabel;

	long remaining; // How many milliseconds remain in the count-down.
	long lastUpdate; // When count was last updated
	Timer timer; // Updates the count every second
	NumberFormat format; // Format minutes:seconds with leading zeros
	private TradingGUI parentApplet;

	public StatusBar(TradingGUI parentApplet) {
		this.parentApplet = parentApplet;
		initComponents();
		setSize(800, 50);
		initTimer();
	}

	private void initTimer() {
		timerLabel.setText("Session time: 00:00");
		timerLabel.setForeground(Color.BLACK);


		// Figure out how long to count for by reading the "minutes" parameter
		// defined in a <param> tag inside the <applet> tag. Convert to ms.
		remaining = 1800000; // 10 minutes by default

		// Obtain a NumberFormat object to convert number of minutes and
		// seconds to strings. Set it up to produce a leading 0 if necessary
		format = NumberFormat.getNumberInstance();
		format.setMinimumIntegerDigits(2); // pad with 0 if necessary

		// Specify a MouseListener to handle mouse events in the applet.
		// Note that the applet implements this interface itself
		addMouseListener(this);

		// Create a timer to call the actionPerformed() method immediately,
		// and then every 1000 milliseconds. Note we don't start the timer yet.
		timer = new Timer(1000, this);
		timer.setInitialDelay(0); // First timer is immediate.
	}

	

	/**
	 *  The browser calls this to start the applet running
	 *  The resume() method is defined below.
	 */
	public void start() { resume(); } // Start displaying updates


	/**
	 * Stop displaying updates
	 * The browser calls this to stop the applet. It may be restarted later. 
	 * The pause() method is defined below
	 */
	public void stop() { pause();  
	timerLabel.setForeground(Color.BLACK);
	timerLabel.setText("Session time: 00:00");

	} 

	/**
	 * Start or resume the count-down
	 */
	private void resume() {
		// Restore the time we're counting down from and restart the timer.
		lastUpdate = System.currentTimeMillis();
		timer.start(); // Start the timer
	}

	public void restart() {
		stop();
		remaining = 1800000;
		timerLabel.setForeground(Color.BLACK);
		// remaining = 40000;
		start();
	}

	/**
	 * Pause the count-down
	 */
	private void pause() {
		// Subtract elapsed time from the remaining time and stop timing
		long now = System.currentTimeMillis();
		remaining -= (now - lastUpdate);
		timer.stop(); // Stop the timer
	}

	
	
	/** 
	 * Update the displayed time. This method is called from actionPerformed()
	 * which is itself invoked by the timer.
	 */
	private void updateDisplay() {
		long now = System.currentTimeMillis(); // current time in ms
		long elapsed = now - lastUpdate; // ms elapsed since last update
		remaining -= elapsed; // adjust remaining time
		lastUpdate = now; // remember this update time

		// Convert remaining milliseconds to mm:ss format and display
		if (remaining < 0) remaining = 0;
		int minutes = (int)(remaining/60000);
		int seconds = (int)((remaining%60000)/1000);
		if (seconds < 30 && minutes == 0) timerLabel.setForeground(Color.red);
		timerLabel.setText("Session time: "+format.format(minutes) + ":" + format.format(seconds));

		// If we've completed the count-down beep and display new page
		if (remaining == 0) {
			// Stop updating now.
			GUILogOutSenderNoReply logOutSender = null;
			try {
				Control controlUser = new Control(parentApplet.getLoggedUser().getUserID(), Enums.ControlType.LOG_OUT_REQUEST,null);
				logOutSender = new GUILogOutSenderNoReply(TradingGUI.IMQ_ADDRESS, Enums.QueueName.LOG_OUT_REQUEST_QUEUE.toString(), parentApplet.getLoggedUser().getUserID()+"-LogoutRequest");
				logOutSender.sendLogOut(controlUser);
				logOutSender.close();
			}
			catch(Exception er) {
				parentApplet.showPopupError(er.getMessage());
			}
			finally {
				try {
					if (logOutSender != null) logOutSender.close();
				}
				catch(Exception er1) {
					parentApplet.showPopupError(er1.getMessage());
				}
			}
			parentApplet.logoutSuccessful();
			timer.stop();
		}
	}

	/**
	 * This method implements the ActionListener interface.
	 * It is invoked once a second by the Timer object
	 * and updates the JLabel to display minutes and seconds remaining.
	 */
	public void actionPerformed(ActionEvent e) { updateDisplay(); }

	/**
	 * The methods below implement the MouseListener interface. We use
	 * two of them to pause the count-down when the mouse hovers over the timer.
	 * Note that we also display a message in the status line
	 */
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}



	/**
	 * Activates the green bubble. :)
	 */
	public void setConnected(String text) {
		setLabelText(text);
		statusIcon.setIcon(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/connected.png"))); // NOI18N
		statusIcon.setText("Connected");
		restart();
	}

	/**
	 * Activates the red bubble.
	 */
	public void setDisconnected(String text) {
		setLabelText(text);
		statusIcon.setIcon(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/disconnected.png"))); // NOI18N
		statusIcon.setText("Disconnected");
		stop();
	}

	/**
	 * Replaces the status text with an empty string.
	 */
	public void removeLabel() {
		statusLabel.setText("");
	}

	/**
	 * Sets the status text.
	 * @param statusText
	 */
	public void setLabelText(String statusText) {
		statusLabel.setText(statusText);
	}

	private void initComponents() {

		statusLabel = new javax.swing.JLabel();
		timerLabel = new javax.swing.JLabel();
		statusIcon = new javax.swing.JLabel();

		setBorder(javax.swing.BorderFactory.createEtchedBorder());
		setLayout(new java.awt.BorderLayout());

		statusLabel.setAlignmentX(5.0F);
		statusLabel.setPreferredSize(new java.awt.Dimension(400, 20));
		add(statusLabel, java.awt.BorderLayout.WEST);

		timerLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		timerLabel.setText("Timer");
		add(timerLabel, java.awt.BorderLayout.CENTER);

		statusIcon.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		statusIcon.setText("Disconnected");
		statusIcon.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
		statusIcon.setPreferredSize(new java.awt.Dimension(120, 14));
		add(statusIcon, java.awt.BorderLayout.EAST);
	}


}
